//==============================================================================
// Island Hopping
// by Felix Hermansson, Tsunami Studios (hoodncloak@hotmail.com)
// 11 November 2006
//
// Random Map Script for Age of Empires III: The War Chiefs
// Partly inspired by the maox map by gangyouxi
//==============================================================================
// History:
// 05-02-06: Original release
// 11-11-06: Updated for TWC (special handling for native civilizations)
//==============================================================================

//==============================================================================
// includes
//==============================================================================

// include mercenaries
include "mercenaries.xs";


//==============================================================================
// function main 
// generates map
//==============================================================================

void main(void)
{
   // Map generation status bar
   rmSetStatusText("",0.01);

   // Check for nomad start
   bool nomad = rmGetNomadStart();

   // Define map size
   int playerTiles = 24000;
   if (cNumberNonGaiaPlayers >4)
   {
      playerTiles = 20000;
   }
   int size = 2.0*sqrt(cNumberNonGaiaPlayers*playerTiles);
   rmSetMapSize(size, size);  

   // Set lighting
   rmSetLightingSet("caribbean");

   // Pick default terrain
   rmSetSeaLevel(1.0);
   rmSetSeaType("caribbean coast");
   rmEnableLocalWater(false);
   rmSetBaseTerrainMix("amazon grass");   
   rmSetMapType("caribbean");
   rmSetMapType("tropical");
   rmSetMapType("water");
   rmSetWorldCircleConstraint(true);

   // Initialize map
   rmTerrainInitialize("water");

   // Choose mercs
   chooseMercs();

   // Map generation status bar
   rmSetStatusText("",0.10);


   // Define constraints to have objects and areas avoid each other

   // Define classes
   int classPlayer = rmDefineClass("player");
   int classIsland = rmDefineClass("island");
   rmDefineClass("classCliff");
   rmDefineClass("classLake");
   rmDefineClass("classPatch");
   rmDefineClass("starting settlement");
   rmDefineClass("startingUnit");
   rmDefineClass("classForest");
   rmDefineClass("importantItem");
   rmDefineClass("natives");
   rmDefineClass("nuggets");
   rmDefineClass("center");

   // Corner constraint
   rmSetWorldCircleConstraint(true);
   
   // Map edge constraints
   int playerEdgeConstraint = rmCreateBoxConstraint("player edge of map", rmXTilesToFraction(10), rmZTilesToFraction(10), 1.0-rmXTilesToFraction(10), 1.0-rmZTilesToFraction(10), 0.01);
   int longPlayerEdgeConstraint = rmCreateBoxConstraint("long avoid edge of map", rmXTilesToFraction(20), rmZTilesToFraction(20), 1.0-rmXTilesToFraction(20), 1.0-rmZTilesToFraction(20), 0.01);
   
   // Center constraints
   int centerConstraint = rmCreateClassDistanceConstraint("stay away from center", rmClassID("center"), rmXFractionToMeters(0.10));
   int centerConstraintFar = rmCreateClassDistanceConstraint("stay away from center far", rmClassID("center"), rmXFractionToMeters(0.23));
   int centerConstraintExtreme = rmCreateClassDistanceConstraint("stay away from center extremely far", rmClassID("center"), rmXFractionToMeters(0.50));

   // Circle constraints
   int circleConstraint = rmCreatePieConstraint("circle constraint", 0.5, 0.5, 0, rmZFractionToMeters(0.44), rmDegreesToRadians(0), rmDegreesToRadians(360));
   int circleConstraintShort = rmCreatePieConstraint("circle constraint short", 0.5, 0.5, 0, rmZFractionToMeters(0.35), rmDegreesToRadians(0), rmDegreesToRadians(360));
   int circleMeterConstraint = rmCreatePieConstraint("circle meter constraint", 0.5, 0.5, 0, ((size/2) - 25), rmDegreesToRadians(0), rmDegreesToRadians(360));

   // Cardinal direction constraints
   int Northward = rmCreatePieConstraint("northMapConstraint", 0.5, 0.5, 0, rmZFractionToMeters(0.5), rmDegreesToRadians(315), rmDegreesToRadians(135));
   int Southward = rmCreatePieConstraint("southMapConstraint", 0.5, 0.5, 0, rmZFractionToMeters(0.5), rmDegreesToRadians(135), rmDegreesToRadians(315));
   int Eastward = rmCreatePieConstraint("eastMapConstraint", 0.5, 0.5, 0, rmZFractionToMeters(0.5), rmDegreesToRadians(45), rmDegreesToRadians(225));
   int Westward = rmCreatePieConstraint("westMapConstraint", 0.5, 0.5, 0, rmZFractionToMeters(0.5), rmDegreesToRadians(225), rmDegreesToRadians(45));

   // Player constraints
   int extremePlayerConstraint = rmCreateClassDistanceConstraint("extreme distance player constraint", classPlayer, 100.0);
   int longPlayerConstraint = rmCreateClassDistanceConstraint("long distance player constraint", classPlayer, 70.0);
   int playerConstraint = rmCreateClassDistanceConstraint("standard distance player constraint", classPlayer, 45.0);
   int shortPlayerConstraint = rmCreateClassDistanceConstraint("short distance player constraint", classPlayer, 20.0);

   // Nature constraints (forests, mines & starting resources)
   int forestObjConstraint = rmCreateTypeDistanceConstraint("forest obj", "all", 6.0);
   int forestConstraint = rmCreateClassDistanceConstraint("forest vs. forest", rmClassID("classForest"), 40.0);
   int avoidCoin = rmCreateTypeDistanceConstraint("avoid coin", "Mine", 40.0);
   int shortAvoidCoin = rmCreateTypeDistanceConstraint("short avoid coin", "gold", 10.0);
   int avoidResource = rmCreateTypeDistanceConstraint("resource avoid resource", "resource", 20.0);
   int avoidStartResource = rmCreateTypeDistanceConstraint("start resource no overlap", "resource", 10.0);

   // Impassable land constraints
   int avoidImpassableLand = rmCreateTerrainDistanceConstraint("avoid impassable land", "Land", false, 6.0);
   int shortAvoidImpassableLand = rmCreateTerrainDistanceConstraint("short avoid impassable land", "Land", false, 2.0);
   int longAvoidImpassableLand = rmCreateTerrainDistanceConstraint("long avoid impassable land", "Land", false, 10.0);

   // Island constraints
   int avoidIslands = rmCreateClassDistanceConstraint("island vs. island", classIsland, 25.0);
   int avoidIslandsShort = rmCreateClassDistanceConstraint("short avoid island", classIsland, 10.0);
   int avoidIslandsLong = rmCreateClassDistanceConstraint("long avoid island", classIsland, 100.0);

   // Unit constraints
   int avoidStartingUnits = rmCreateClassDistanceConstraint("objects avoid starting units", rmClassID("startingUnit"), 45.0);
   int shortAvoidStartingUnits = rmCreateClassDistanceConstraint("objects avoid starting units short", rmClassID("startingUnit"), 10.0);
   int avoidImportantItem = rmCreateClassDistanceConstraint("general items avoid each other", rmClassID("importantItem"), 10.0);
   int avoidNatives = rmCreateClassDistanceConstraint("objects avoid natives", rmClassID("natives"), 30.0);
   int avoidNativesLong = rmCreateClassDistanceConstraint("objects avoid natives long", rmClassID("natives"), 70.0);
   int avoidNativesExtreme = rmCreateClassDistanceConstraint("objects avoid natives extreme", rmClassID("natives"), 100.0);
   int avoidPirates = rmCreateTypeDistanceConstraint("avoid pirates", "privateer", 75.0);
   int avoidNuggets = rmCreateClassDistanceConstraint("objects avoid nuggets", rmClassID("nuggets"), 60.0);
   int shortNuggetConstraint = rmCreateTypeDistanceConstraint("avoid nugget objects", "AbstractNugget", 7.0);
   int turkeyConstraint = rmCreateTypeDistanceConstraint("avoid turkeys", "turkey", 40.0);
   int capybaraConstraint = rmCreateTypeDistanceConstraint("avoid capybaras", "capybara", 40.0);
   int fishConstraint = rmCreateTypeDistanceConstraint("avoid fish", "abstractFish", 10.0);

   // General constraint (used mostly for decoration objects)
   int avoidAll = rmCreateTypeDistanceConstraint("avoid all", "all", 6.0);

   // Map generation status bar
   rmSetStatusText("",0.20);


   // Create islands

   // Place players in a rough circle towards the outskirts of the map
   rmSetTeamSpacingModifier(0.9);
   rmPlacePlayersCircular(0.35, 0.40, rmDegreesToRadians(360/(cNumberNonGaiaPlayers*3)));

   // Create player islands
   float playerFraction = rmAreaTilesToFraction(1000);
   for(i=1; <cNumberPlayers)
   {
      int id = rmCreateArea("Player"+i);
      rmSetPlayerArea(i, id);
      rmSetAreaSize(id, playerFraction, playerFraction);
      rmAddAreaToClass(id, classPlayer);
      rmSetAreaMix(id, "caribbean grass");
      rmAddAreaToClass(id, classIsland);
      rmAddAreaConstraint(id, avoidIslands);
      rmAddAreaConstraint(id, circleMeterConstraint);
      rmSetAreaCoherence(id, 0.6);
      rmSetAreaSmoothDistance(id, 10);
      rmSetAreaBaseHeight(id, 3.0);
      rmSetAreaElevationType(id, cElevTurbulence);
      rmSetAreaElevationVariation(id, 3.0);
      rmSetAreaElevationMinFrequency(id, 0.09);
      rmSetAreaElevationOctaves(id, 3);
      rmSetAreaElevationPersistence(id, 0.2);
      rmSetAreaLocPlayer(id, i);
      rmSetAreaWarnFailure(id, false);
   }

   // Build all player areas
   rmBuildAllAreas();


   // Create additional islands

   // Define number of islands
   int islandCount = -1;
   int islandFailCount = -1;

   islandCount = size / 10;

   // Place islands
   for (i=0; <islandCount)
   {   
      int islandID = rmCreateArea("island"+i);
      rmSetAreaWarnFailure(islandID, false);
      rmSetAreaSize(islandID, rmAreaTilesToFraction(500), rmAreaTilesToFraction(1000));
      rmSetAreaMix(islandID, "caribbean grass");
      rmAddAreaToClass(islandID, classIsland);
      rmAddAreaConstraint(islandID, avoidIslands);
      rmAddAreaConstraint(islandID, circleMeterConstraint);
      rmSetAreaCoherence(islandID, 0.3);
      rmSetAreaSmoothDistance(islandID, 10);
      rmSetAreaBaseHeight(islandID, 3.0);
      rmSetAreaElevationType(islandID, cElevTurbulence);
      rmSetAreaElevationVariation(islandID, 3.0);
      rmSetAreaElevationMinFrequency(islandID, 0.09);
      rmSetAreaElevationOctaves(islandID, 3);
      rmSetAreaElevationPersistence(islandID, 0.2);
      if(rmBuildArea(islandID) == false)
      {
         // Stop placement once it fails 5 times in a row 
         islandFailCount++;
         if(islandFailCount == 5)
         {      
		break;
         }
      }
      else
      {
         islandFailCount = 0;
      }
   }

   // Map generation status bar
   rmSetStatusText("",0.30);


   // Define and place starting units and resources

   // Define player units
   if (nomad == false)
   {
      // Define TC
      int startingTCID = rmCreateObjectDef("startingTC");
      rmAddObjectDefItem(startingTCID, "townCenter", 1, 5.0);
      rmAddObjectDefConstraint(startingTCID, avoidImpassableLand);
      rmAddObjectDefToClass(startingTCID, rmClassID("player"));

      // Define starting units (explorer, war chief, etc.)
      int startingUnits = rmCreateStartingUnitsObjectDef(5.0);
      rmAddObjectDefConstraint(startingUnits, avoidImpassableLand);
      rmSetObjectDefMinDistance(startingUnits, 10.0);
      rmSetObjectDefMaxDistance(startingUnits, 12.0);
      rmAddObjectDefToClass(startingUnits, rmClassID("startingUnit"));
   }

   // Define water spawn flag
   int waterSpawnFlagID = rmCreateObjectDef("water spwan flag");
   rmAddObjectDefItem(waterSpawnFlagID, "HomeCityWaterSpawnFlag", 1, 5.0);
   rmAddObjectDefConstraint(waterSpawnFlagID, circleConstraintShort);
   rmSetObjectDefMinDistance(waterSpawnFlagID, 20.0);
   rmSetObjectDefMaxDistance(waterSpawnFlagID, 40.0);
   rmAddObjectDefToClass(waterSpawnFlagID, rmClassID("startingUnit"));

   // Define starting caravel
   int caravelID = rmCreateObjectDef("starting caravel");
   rmAddObjectDefItem(caravelID, "caravel", 1, 5.0);
   rmAddObjectDefConstraint(caravelID, circleConstraintShort);
   rmSetObjectDefMinDistance(caravelID, 20.0);
   rmSetObjectDefMaxDistance(caravelID, 40.0);
   rmAddObjectDefToClass(caravelID, rmClassID("startingUnit"));
   rmAddObjectDefConstraint(caravelID, shortAvoidStartingUnits);
   if (nomad == true)
   {
      rmSetObjectDefGarrisonStartingUnits(caravelID, true);
   }

   // Define starting galley
   int galleyID = rmCreateObjectDef("starting galley");
   rmAddObjectDefItem(galleyID, "Galley", 1, 5.0);
   rmAddObjectDefConstraint(galleyID, circleConstraint);
   rmSetObjectDefMinDistance(galleyID, 20.0);
   rmSetObjectDefMaxDistance(galleyID, 40.0);
   rmAddObjectDefToClass(galleyID, rmClassID("startingUnit"));
   rmAddObjectDefConstraint(galleyID, shortAvoidStartingUnits);
   if (nomad == true)
   {
      rmSetObjectDefGarrisonStartingUnits(galleyID, true);
   }

   // Define starting war canoes
   int canoe1ID = rmCreateObjectDef("starting war canoe 1");
   rmAddObjectDefItem(canoe1ID, "xpWarCanoe", 1, 5.0);
   rmAddObjectDefConstraint(canoe1ID, circleConstraint);
   rmSetObjectDefMinDistance(canoe1ID, 20.0);
   rmSetObjectDefMaxDistance(canoe1ID, 40.0);
   rmAddObjectDefToClass(canoe1ID, rmClassID("startingUnit"));
   rmAddObjectDefConstraint(canoe1ID, shortAvoidStartingUnits);
   if (nomad == true)
   {
      rmSetObjectDefGarrisonStartingUnits(canoe1ID, true);
   }
   int canoe2ID = rmCreateObjectDef("starting war canoe 2");
   rmAddObjectDefItem(canoe2ID, "xpWarCanoe", 1, 5.0);
   rmAddObjectDefConstraint(canoe2ID, circleConstraint);
   rmSetObjectDefMinDistance(canoe2ID, 20.0);
   rmSetObjectDefMaxDistance(canoe2ID, 40.0);
   rmAddObjectDefToClass(canoe2ID, rmClassID("startingUnit"));
   rmAddObjectDefConstraint(canoe2ID, shortAvoidStartingUnits);

   // Define starting mine
   int startSilver1ID = rmCreateObjectDef("player silver close");
   rmAddObjectDefItem(startSilver1ID, "mine", 1, 5.0);
   rmSetObjectDefMinDistance(startSilver1ID, 12.0);
   rmSetObjectDefMaxDistance(startSilver1ID, 20.0);
   rmAddObjectDefConstraint(startSilver1ID, avoidStartResource);
   rmAddObjectDefConstraint(startSilver1ID, avoidImpassableLand);

   // Define starting turkeys
   int startTurkeyID = rmCreateObjectDef("starting turkeys");
   rmAddObjectDefItem(startTurkeyID, "turkey", rmRandInt(3,4), 8.0);
   rmSetObjectDefMinDistance(startTurkeyID, 12);
   rmSetObjectDefMaxDistance(startTurkeyID, 24);
   rmAddObjectDefConstraint(startTurkeyID, avoidStartResource);
   rmAddObjectDefConstraint(startTurkeyID, shortAvoidImpassableLand);
   rmSetObjectDefCreateHerd(startTurkeyID, true);

   // Define starting trees
   int startAreaTreeID = rmCreateObjectDef("starting trees");
   rmAddObjectDefItem(startAreaTreeID, "treeCaribbean", rmRandInt(2,3), 8.0);
   rmSetObjectDefMinDistance(startAreaTreeID, 12);
   rmSetObjectDefMaxDistance(startAreaTreeID, 24);
   rmAddObjectDefConstraint(startAreaTreeID, avoidStartResource);
   rmAddObjectDefConstraint(startAreaTreeID, shortAvoidImpassableLand);
   rmAddObjectDefConstraint(startAreaTreeID, shortAvoidStartingUnits);

   // Place starting units and resources for all payers 
   for(i=1; <cNumberPlayers)
   {
      if (nomad == false)
      {
         rmPlaceObjectDefAtLoc(startingTCID, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      }
      rmPlaceObjectDefAtLoc(waterSpawnFlagID, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      if (rmGetPlayerCiv(i) == rmGetCivID("Ottomans"))
      {
         rmPlaceObjectDefAtLoc(galleyID, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      }
      else if ((rmGetPlayerCiv(i) == rmGetCivID("XPAztec")) ||
               (rmGetPlayerCiv(i) == rmGetCivID("XPIroquois")) ||
               (rmGetPlayerCiv(i) == rmGetCivID("XPSioux")))
      {
         rmPlaceObjectDefAtLoc(canoe1ID, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
         rmPlaceObjectDefAtLoc(canoe2ID, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      }
      else
      {
         rmPlaceObjectDefAtLoc(caravelID, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      }
      rmPlaceObjectDefAtLoc(startSilver1ID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      if (nomad == false)
      {
         rmPlaceObjectDefAtLoc(startingUnits, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
         rmPlaceObjectDefAtLoc(startTurkeyID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
         rmPlaceObjectDefAtLoc(startTurkeyID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      }
      rmPlaceObjectDefAtLoc(startAreaTreeID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startAreaTreeID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startAreaTreeID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
   }

   // Map generation status bar
   rmSetStatusText("",0.40);


   // Define and place natives (Caribs only)

   // Define natives per village
   int subCiv0 = -1;
   int subCiv1 = -1;
   int subCiv2 = -1;
   int subCiv3 = -1;
   subCiv0 = rmGetCivID("Caribs");
   rmSetSubCiv(0, "Caribs", true);
   subCiv1 = rmGetCivID("Caribs");
   rmSetSubCiv(1, "Caribs", true);
   if (cNumberNonGaiaPlayers > 4)
   {
      subCiv2 = rmGetCivID("Caribs");
      rmSetSubCiv(2, "Caribs", true);
   }
   if (cNumberNonGaiaPlayers > 6)
   {
      subCiv3 = rmGetCivID("Caribs");
      rmSetSubCiv(3, "Caribs", true);
   }

   // Define and place native villages
   int nativeVillage1ID = -1;
   int nativeVillage2ID = -1;
   int nativeVillage3ID = -1;
   int nativeVillage4ID = -1;
   if (subCiv0 == rmGetCivID("Caribs"))
   {
      // Native village 1
      int nativeVillage1Type = rmRandInt(1,5);
      nativeVillage1ID = rmCreateGrouping("native village 1", "native carib village "+nativeVillage1Type);
      rmSetGroupingMinDistance(nativeVillage1ID, 0.0);
      rmSetGroupingMaxDistance(nativeVillage1ID, rmXFractionToMeters(0.25));
      rmAddGroupingConstraint(nativeVillage1ID, extremePlayerConstraint);
      rmAddGroupingConstraint(nativeVillage1ID, avoidNativesExtreme);
      rmAddGroupingConstraint(nativeVillage1ID, longAvoidImpassableLand);
      rmAddGroupingToClass(nativeVillage1ID, rmClassID("natives"));
      rmAddGroupingToClass(nativeVillage1ID, rmClassID("importantItem"));
      rmPlaceGroupingAtLoc(nativeVillage1ID, 0, 0.5, 0.5);
   }
   if (subCiv1 == rmGetCivID("Caribs"))
   {
      // Native village 2
      int nativeVillage2Type = rmRandInt(1,5);
      nativeVillage2ID = rmCreateGrouping("native village 2", "native carib village "+nativeVillage2Type);
      rmSetGroupingMinDistance(nativeVillage2ID, 0.0);
      rmSetGroupingMaxDistance(nativeVillage2ID, rmXFractionToMeters(0.25));
      rmAddGroupingConstraint(nativeVillage2ID, extremePlayerConstraint);
      rmAddGroupingConstraint(nativeVillage2ID, avoidNativesExtreme);
      rmAddGroupingConstraint(nativeVillage2ID, longAvoidImpassableLand);
      rmAddGroupingToClass(nativeVillage2ID, rmClassID("natives"));
      rmAddGroupingToClass(nativeVillage2ID, rmClassID("importantItem"));
      rmPlaceGroupingAtLoc(nativeVillage2ID, 0, 0.5, 0.5);
   }
   if (subCiv2 == rmGetCivID("Caribs"))
   {
      // Native village 3
      int nativeVillage3Type = rmRandInt(1,5);
      nativeVillage3ID = rmCreateGrouping("native village 3", "native carib village "+nativeVillage3Type);
      rmSetGroupingMinDistance(nativeVillage3ID, 0.0);
      rmSetGroupingMaxDistance(nativeVillage3ID, rmXFractionToMeters(0.25));
      rmAddGroupingConstraint(nativeVillage3ID, extremePlayerConstraint);
      rmAddGroupingConstraint(nativeVillage3ID, avoidNativesExtreme);
      rmAddGroupingConstraint(nativeVillage3ID, longAvoidImpassableLand);
      rmAddGroupingToClass(nativeVillage3ID, rmClassID("natives"));
      rmAddGroupingToClass(nativeVillage3ID, rmClassID("importantItem"));
      rmPlaceGroupingAtLoc(nativeVillage3ID, 0, 0.5, 0.5);
   }
   if (subCiv1 == rmGetCivID("Caribs"))
   {
      // Native village 4
      int nativeVillage4Type = rmRandInt(1,5);
      nativeVillage4ID = rmCreateGrouping("native village 4", "native carib village "+nativeVillage4Type);
      rmSetGroupingMinDistance(nativeVillage4ID, 0.0);
      rmSetGroupingMaxDistance(nativeVillage4ID, rmXFractionToMeters(0.25));
      rmAddGroupingConstraint(nativeVillage4ID, extremePlayerConstraint);
      rmAddGroupingConstraint(nativeVillage4ID, avoidNativesExtreme);
      rmAddGroupingConstraint(nativeVillage4ID, longAvoidImpassableLand);
      rmAddGroupingToClass(nativeVillage4ID, rmClassID("natives"));
      rmAddGroupingToClass(nativeVillage4ID, rmClassID("importantItem"));
      rmPlaceGroupingAtLoc(nativeVillage4ID, 0, 0.5, 0.5);
   }

   // Map generation status bar
   rmSetStatusText("",0.50);


   // Define and place pirates
   int pirateCount = 2;
   pirateCount = rmRandInt(2,cNumberNonGaiaPlayers);

   int pirateShipID = rmCreateObjectDef("pirate ship");
   rmAddObjectDefItem(pirateShipID, "privateer", 1, 0.0);
   rmSetObjectDefMinDistance(pirateShipID, 0.0);
   rmSetObjectDefMaxDistance(pirateShipID, rmXFractionToMeters(0.2));
   rmAddObjectDefConstraint(pirateShipID, extremePlayerConstraint);
   rmAddObjectDefConstraint(pirateShipID, avoidPirates);

   for(i=0; <pirateCount)
   { 
      rmPlaceObjectDefAtLoc(pirateShipID, 0, 0.5, 0.5);
   }

   // Map generation status bar
   rmSetStatusText("",0.60);


   // Place silver mines and forests

   // Place additional silver mines
   int silverCount = cNumberNonGaiaPlayers*8;
   int silverMineID = rmCreateObjectDef("silver mines");
   rmAddObjectDefItem(silverMineID, "mine", 1, 0.0);
   rmSetObjectDefMinDistance(silverMineID, 0.0);
   rmSetObjectDefMaxDistance(silverMineID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(silverMineID, avoidImpassableLand);
   rmAddObjectDefConstraint(silverMineID, avoidCoin);
   rmAddObjectDefConstraint(silverMineID, longPlayerConstraint);

   rmAddObjectDefConstraint(silverMineID, avoidAll);
   for(i=0; <silverCount)
   { 
      rmPlaceObjectDefAtLoc(silverMineID, 0, 0.5, 0.5);
   }


   // Place forests
   int forestCount = -1;
   int failCount = -1;
   int forestTreeID = 0;
   forestCount = size;
   failCount = 0;
   for (i=0; <forestCount)  // Northern forests
   {   
      int northForest=rmCreateArea("northforest"+i);
      rmSetAreaWarnFailure(northForest, false);
      rmSetAreaSize(northForest, rmAreaTilesToFraction(20), rmAreaTilesToFraction(100));
      rmSetAreaForestType(northForest, "caribbean palm forest");
      rmSetAreaForestDensity(northForest, 0.3);
      rmAddAreaToClass(northForest, rmClassID("classForest"));
      rmSetAreaForestClumpiness(northForest, 0.0);
      rmSetAreaForestUnderbrush(northForest, 0.0);
      rmSetAreaBaseHeight(northForest, 2.0);
      rmSetAreaMinBlobs(northForest, 1);
      rmSetAreaMaxBlobs(northForest, 3);
      rmSetAreaMinBlobDistance(northForest, 16.0);
      rmSetAreaMaxBlobDistance(northForest, 30.0);
      rmSetAreaCoherence(northForest, 0.7);
      rmSetAreaSmoothDistance(northForest, 1);
      rmAddAreaConstraint(northForest, avoidImportantItem);
      rmAddAreaConstraint(northForest, shortAvoidImpassableLand);
      rmAddAreaConstraint(northForest, shortAvoidCoin);
      if (nomad == false)
      {
        rmAddAreaConstraint(northForest, shortPlayerConstraint);
      }
      rmAddAreaConstraint(northForest, Northward);
      if(rmBuildArea(northForest) == false)
      {
         // Stop placement once it fails 5 times in a row 
         failCount++;
         if(failCount == 5)
         {      
		break;
         }
      }
      else
      {
         failCount = 0;
      }
   }
   failCount = 0;
   for (i=0; <forestCount)  // Southern forests
   {   
      int southForest=rmCreateArea("southforest"+i);
      rmSetAreaWarnFailure(southForest, false);
      rmSetAreaSize(southForest, rmAreaTilesToFraction(20), rmAreaTilesToFraction(100));
      rmSetAreaForestType(southForest, "caribbean palm forest");
      rmSetAreaForestDensity(southForest, 0.3);
      rmAddAreaToClass(southForest, rmClassID("classForest"));
      rmSetAreaForestClumpiness(southForest, 0.0);
      rmSetAreaForestUnderbrush(southForest, 0.0);
      rmSetAreaBaseHeight(southForest, 2.0);
      rmSetAreaMinBlobs(southForest, 1);
      rmSetAreaMaxBlobs(southForest, 3);
      rmSetAreaMinBlobDistance(southForest, 16.0);
      rmSetAreaMaxBlobDistance(southForest, 30.0);
      rmSetAreaCoherence(southForest, 0.7);
      rmSetAreaSmoothDistance(southForest, 1);
      rmAddAreaConstraint(southForest, avoidImportantItem);
      rmAddAreaConstraint(southForest, shortAvoidImpassableLand);
      rmAddAreaConstraint(southForest, shortAvoidCoin);
      if (nomad == false)
      {
        rmAddAreaConstraint(southForest, shortPlayerConstraint);
      }
      rmAddAreaConstraint(southForest, Southward);
      if(rmBuildArea(southForest)==false)
      {
         // Stop placement once it fails 5 times in a row 
         failCount++;
         if(failCount == 5)
         {      
		break;
         }
      }
      else
      {
         failCount = 0;
      }
   }

   // Map generation status bar
   rmSetStatusText("",0.70);


   // Place additional animals

   // Place turkeys
   int turkeyCount = rmRandInt(4,6);
   int turkeyHerdCount = cNumberNonGaiaPlayers + rmRandInt(1,cNumberNonGaiaPlayers);
   int turkeyNorthID = rmCreateObjectDef("turkey flock north");
   rmAddObjectDefItem(turkeyNorthID, "turkey", turkeyCount, 5.0);
   rmSetObjectDefMinDistance(turkeyNorthID, 0.0);
   rmSetObjectDefMaxDistance(turkeyNorthID, rmXFractionToMeters(0.50));
   rmAddObjectDefConstraint(turkeyNorthID, avoidImpassableLand);
   rmAddObjectDefConstraint(turkeyNorthID, shortAvoidCoin);
   if (nomad == false)
   {
      rmAddObjectDefConstraint(turkeyNorthID, playerConstraint);
   }
   rmAddObjectDefConstraint(turkeyNorthID, turkeyConstraint);
   rmAddObjectDefConstraint(turkeyNorthID, Northward);
   rmSetObjectDefCreateHerd(turkeyNorthID, true);
   for(i=0; <turkeyHerdCount)
   { 
      rmPlaceObjectDefAtLoc(turkeyNorthID, 0, 0.5, 0.5); // northern half of the map
   }
   int turkeySouthID = rmCreateObjectDef("turkey flock south");
   rmAddObjectDefItem(turkeySouthID, "turkey", turkeyCount, 5.0);
   rmSetObjectDefMinDistance(turkeySouthID, 0.0);
   rmSetObjectDefMaxDistance(turkeySouthID, rmXFractionToMeters(0.50));
   rmAddObjectDefConstraint(turkeySouthID, avoidImpassableLand);
   rmAddObjectDefConstraint(turkeySouthID, shortAvoidCoin);
   if (nomad == false)
   {
      rmAddObjectDefConstraint(turkeySouthID, playerConstraint);
   }
   rmAddObjectDefConstraint(turkeySouthID, turkeyConstraint);
   rmAddObjectDefConstraint(turkeySouthID, Southward);
   rmSetObjectDefCreateHerd(turkeySouthID, true);
   for(i=0; <turkeyHerdCount)
   { 
      rmPlaceObjectDefAtLoc(turkeySouthID, 0, 0.5, 0.5); // Southern half of the map
   }

   // Place capybaras
   int capybaraCount = rmRandInt(3,5);
   int capybaraHerdCount = cNumberNonGaiaPlayers + rmRandInt(1,cNumberNonGaiaPlayers);
   int capybaraNorthID = rmCreateObjectDef("capybara herd north");
   rmAddObjectDefItem(capybaraNorthID, "capybara", capybaraCount, 5.0);
   rmSetObjectDefMinDistance(capybaraNorthID, 0.0);
   rmSetObjectDefMaxDistance(capybaraNorthID, rmXFractionToMeters(0.50));
   rmAddObjectDefConstraint(capybaraNorthID, avoidImpassableLand);
   rmAddObjectDefConstraint(capybaraNorthID, shortAvoidCoin);
   if (nomad == false)
   {
      rmAddObjectDefConstraint(capybaraNorthID, playerConstraint);
   }
   rmAddObjectDefConstraint(capybaraNorthID, turkeyConstraint);
   rmAddObjectDefConstraint(capybaraNorthID, capybaraConstraint);
   rmAddObjectDefConstraint(capybaraNorthID, Northward);
   rmSetObjectDefCreateHerd(capybaraNorthID, true);
   for(i=0; <capybaraHerdCount)
   { 
      rmPlaceObjectDefAtLoc(capybaraNorthID, 0, 0.5, 0.5); // northern half of the map
   }
   int capybaraSouthID = rmCreateObjectDef("capybara herd south");
   rmAddObjectDefItem(capybaraSouthID, "capybara", capybaraCount, 5.0);
   rmSetObjectDefMinDistance(capybaraSouthID, 0.0);
   rmSetObjectDefMaxDistance(capybaraSouthID, rmXFractionToMeters(0.50));
   rmAddObjectDefConstraint(capybaraSouthID, avoidImpassableLand);
   rmAddObjectDefConstraint(capybaraSouthID, shortAvoidCoin);
   if (nomad == false)
   {
      rmAddObjectDefConstraint(capybaraSouthID, playerConstraint);
   }
   rmAddObjectDefConstraint(capybaraSouthID, turkeyConstraint);
   rmAddObjectDefConstraint(capybaraSouthID, capybaraConstraint);
   rmAddObjectDefConstraint(capybaraSouthID, Southward);
   rmSetObjectDefCreateHerd(capybaraSouthID, true);
   for(i=0; <capybaraHerdCount)
   { 
      rmPlaceObjectDefAtLoc(capybaraSouthID, 0, 0.5, 0.5); // Southern half of the map
   }

   // Place mahi mahi
   int mahiCount = 12*cNumberNonGaiaPlayers;
   int mahiNorthID = rmCreateObjectDef("mahi mahi north");
   rmAddObjectDefItem(mahiNorthID, "FishMahi", 1, 5.0);
   rmSetObjectDefMinDistance(mahiNorthID, 0.0);
   rmSetObjectDefMaxDistance(mahiNorthID, rmXFractionToMeters(0.50));
   rmAddObjectDefConstraint(mahiNorthID, fishConstraint);
   rmAddObjectDefConstraint(mahiNorthID, Northward);
   for(i=0; <mahiCount)
   { 
      rmPlaceObjectDefAtLoc(mahiNorthID, 0, 0.5, 0.5); // northern half of the map
   }
   int mahiSouthID = rmCreateObjectDef("mahi mahi south");
   rmAddObjectDefItem(mahiSouthID, "FishMahi", 1, 5.0);
   rmSetObjectDefMinDistance(mahiSouthID, 0.0);
   rmSetObjectDefMaxDistance(mahiSouthID, rmXFractionToMeters(0.50));
   rmAddObjectDefConstraint(mahiSouthID, fishConstraint);
   rmAddObjectDefConstraint(mahiSouthID, Southward);
   for(i=0; <mahiCount)
   { 
      rmPlaceObjectDefAtLoc(mahiSouthID, 0, 0.5, 0.5); // Southern half of the map
   }

   // Place tarpon
   int tarponCount = 12*cNumberNonGaiaPlayers;
   int tarponNorthID = rmCreateObjectDef("tarpon north");
   rmAddObjectDefItem(tarponNorthID, "FishTarpon", 1, 5.0);
   rmSetObjectDefMinDistance(tarponNorthID, 0.0);
   rmSetObjectDefMaxDistance(tarponNorthID, rmXFractionToMeters(0.50));
   rmAddObjectDefConstraint(tarponNorthID, fishConstraint);
   rmAddObjectDefConstraint(tarponNorthID, Northward);
   for(i=0; <tarponCount)
   { 
      rmPlaceObjectDefAtLoc(tarponNorthID, 0, 0.5, 0.5); // northern half of the map
   }
   int tarponSouthID = rmCreateObjectDef("tarpon south");
   rmAddObjectDefItem(tarponSouthID, "FishTarpon", 1, 5.0);
   rmSetObjectDefMinDistance(tarponSouthID, 0.0);
   rmSetObjectDefMaxDistance(tarponSouthID, rmXFractionToMeters(0.50));
   rmAddObjectDefConstraint(tarponSouthID, fishConstraint);
   rmAddObjectDefConstraint(tarponSouthID, Southward);
   for(i=0; <tarponCount)
   { 
      rmPlaceObjectDefAtLoc(tarponSouthID, 0, 0.5, 0.5); // Southern half of the map
   }

   // Map generation status bar
   rmSetStatusText("",0.80);


   // Define and place Nuggets

   // Medium nuggets (4 per player, outer parts of the map)
   int nugget1 = rmCreateObjectDef("nugget medium"); 
   rmAddObjectDefItem(nugget1, "Nugget", 1, 0.0);
   rmSetNuggetDifficulty(2,2);
   rmAddObjectDefToClass(nugget1, rmClassID("nuggets"));
   rmSetObjectDefMinDistance(nugget1, rmXFractionToMeters(0.30));
   rmSetObjectDefMaxDistance(nugget1, rmXFractionToMeters(0.50));
   rmAddObjectDefConstraint(nugget1, shortPlayerConstraint);
   rmAddObjectDefConstraint(nugget1, avoidImpassableLand);
   rmAddObjectDefConstraint(nugget1, avoidNuggets);
   rmAddObjectDefConstraint(nugget1, circleConstraint);
   rmAddObjectDefConstraint(nugget1, avoidAll);
   rmPlaceObjectDefAtLoc(nugget1, 0, 0.5, 0.5, 4*cNumberNonGaiaPlayers);

   // Hard nuggets (3 per player, inner parts of the map)
   int nugget2 = rmCreateObjectDef("nugget harder"); 
   rmAddObjectDefItem(nugget2, "Nugget", 1, 0.0);
   rmSetNuggetDifficulty(3,3);
   rmAddObjectDefToClass(nugget2, rmClassID("nuggets"));
   rmSetObjectDefMinDistance(nugget2, 0.0);
   rmSetObjectDefMaxDistance(nugget2, rmXFractionToMeters(0.30));
   rmAddObjectDefConstraint(nugget2, shortPlayerConstraint);
   rmAddObjectDefConstraint(nugget2, avoidImpassableLand);
   rmAddObjectDefConstraint(nugget2, avoidNuggets);
   rmAddObjectDefConstraint(nugget2, circleConstraint);
   rmAddObjectDefConstraint(nugget2, avoidAll);
   rmPlaceObjectDefAtLoc(nugget2, 0, 0.5, 0.5, 3*cNumberNonGaiaPlayers);

   // Map generation status bar
   rmSetStatusText("",0.9);


   // Define triggers for nomad style game

   if (nomad == true)
   {
      // Grant all players 600 wood
      for (i = 1; <=cNumberNonGaiaPlayers)
      {
         //Create all triggers first
         rmCreateTrigger("GrantWood"+i);
         rmSwitchToTrigger(rmTriggerID("GrantWood"+i));
         rmSetTriggerActive(true);
         rmSetTriggerLoop(false);
         rmSetTriggerRunImmediately(true);
         rmSetTriggerPriority(4);
         rmAddTriggerEffect("Grant Resources");
         rmSetTriggerEffectParamInt("PlayerID", i);
         rmSetTriggerEffectParam("ResName", "wood" );
         rmSetTriggerEffectParamInt("Amount", 600);
      }  
   }

   // Map generation status bar
   rmSetStatusText("",1.0);
}  
