//==============================================================================
// King of the Valleys
// by Felix Hermansson, Tsunami Studios (hoodncloak@hotmail.com)
// 11 November 2006
//
// Random Map Script for Age of Empires III: The War Chiefs
//==============================================================================
// History:
// 15-01-06: Original release
// 11-11-06: Updated for TWC (special handling for native civilizations)
//==============================================================================


//==============================================================================
// includes
//==============================================================================

// include mercenaries
include "mercenaries.xs";


//==============================================================================
// function main 
// generates map
//==============================================================================

void main(void)
{
   // Map generation status bar
   rmSetStatusText("",0.01);

   // Check for nomad start (additional buildings)
   bool nomad = rmGetNomadStart();

   // Define map size
   int playerTiles = 22000;
   if (cNumberNonGaiaPlayers > 5)
   {
      playerTiles = 20000;
   }
   int size = 2.0 * sqrt(cNumberNonGaiaPlayers * playerTiles);
   rmSetMapSize(size, size);  

   // pick default terrain
   rmEnableLocalWater(false);
   rmSetBaseTerrainMix("rockies_grass");
   rmTerrainInitialize("rockies\ground3_roc", 1.0);
   rmSetMapType("rockies");
   rmSetMapType("land");
   rmSetMapType("grass");

   // Set lighting
   rmSetLightingSet("rockies");

   // Set weather
   rmSetWindMagnitude(2.0);

   // Set map elevation parameters
   rmSetMapElevationParameters(cElevTurbulence, 0.3, 6, 0.4, 7.0);
   rmSetMapElevationHeightBlend(0.5);

   // Choose mercenaries
   chooseMercs();

   // Map generation status bar
   rmSetStatusText("",0.10);


   // Define constraints to have objects and areas avoid each other

   // Define necessary classes
   int classPlayer = rmDefineClass("player");
   rmDefineClass("startingUnit");
   rmDefineClass("classForest");
   rmDefineClass("importantItem");
   rmDefineClass("natives");
   rmDefineClass("nuggets");
   rmDefineClass("center");

   // Corner constraint
   rmSetWorldCircleConstraint(true);
   
   // Map edge constraints
   int playerEdgeConstraint = rmCreateBoxConstraint("player edge of map", rmXTilesToFraction(10), rmZTilesToFraction(10), 1.0-rmXTilesToFraction(10), 1.0-rmZTilesToFraction(10), 0.01);
   int longPlayerEdgeConstraint = rmCreateBoxConstraint("long avoid edge of map", rmXTilesToFraction(20), rmZTilesToFraction(20), 1.0-rmXTilesToFraction(20), 1.0-rmZTilesToFraction(20), 0.01);
   
   // Center constraints
   int centerConstraint = rmCreateClassDistanceConstraint("stay away from center", rmClassID("center"), rmXFractionToMeters(0.10));
   int centerConstraintFar = rmCreateClassDistanceConstraint("stay away from center far", rmClassID("center"), rmXFractionToMeters(0.23));
   int centerConstraintExtreme = rmCreateClassDistanceConstraint("stay away from center extremely far", rmClassID("center"), rmXFractionToMeters(0.50));

   // General circle constraint
   int circleConstraint = rmCreatePieConstraint("circle Constraint", 0.5, 0.5, 0, rmZFractionToMeters(0.47), rmDegreesToRadians(0), rmDegreesToRadians(360));

   // Cardinal direction constraints
   int Northward = rmCreatePieConstraint("northMapConstraint", 0.5, 0.5, 0, rmZFractionToMeters(0.5), rmDegreesToRadians(315), rmDegreesToRadians(135));
   int Southward = rmCreatePieConstraint("southMapConstraint", 0.5, 0.5, 0, rmZFractionToMeters(0.5), rmDegreesToRadians(135), rmDegreesToRadians(315));
   int Eastward = rmCreatePieConstraint("eastMapConstraint", 0.5, 0.5, 0, rmZFractionToMeters(0.5), rmDegreesToRadians(45), rmDegreesToRadians(225));
   int Westward = rmCreatePieConstraint("westMapConstraint", 0.5, 0.5, 0, rmZFractionToMeters(0.5), rmDegreesToRadians(225), rmDegreesToRadians(45));

   // Map quadrant constraints
   int Northeast = rmCreatePieConstraint("northeastMapConstraint", 0.5, 0.5, 0, rmZFractionToMeters(0.5), rmDegreesToRadians(45), rmDegreesToRadians(135));
   int Southeast = rmCreatePieConstraint("southeastMapConstraint", 0.5, 0.5, 0, rmZFractionToMeters(0.5), rmDegreesToRadians(135), rmDegreesToRadians(225));
   int Southwest = rmCreatePieConstraint("southwestMapConstraint", 0.5, 0.5, 0, rmZFractionToMeters(0.5), rmDegreesToRadians(225), rmDegreesToRadians(315));
   int Northwest = rmCreatePieConstraint("northwestMapConstraint", 0.5, 0.5, 0, rmZFractionToMeters(0.5), rmDegreesToRadians(315), rmDegreesToRadians(45));

   // Player constraints
   int extremePlayerConstraint = rmCreateClassDistanceConstraint("extreme distance player constraint", classPlayer, 100.0);
   int longPlayerConstraint = rmCreateClassDistanceConstraint("long distance player constraint", classPlayer, 70.0);
   int playerConstraint = rmCreateClassDistanceConstraint("standard distance player constraint", classPlayer, 45.0);
   int shortPlayerConstraint = rmCreateClassDistanceConstraint("short distance player constraint", classPlayer, 20.0);

   // Nature constraints (forests, mines & starting resources)
   int forestConstraint = rmCreateClassDistanceConstraint("forest vs. forest", rmClassID("classForest"), 40.0);
   int forestConstraintShort = rmCreateClassDistanceConstraint("avoid forest", rmClassID("classForest"), 10.0);
   int avoidGold = rmCreateTypeDistanceConstraint("avoid gold", "minegold", 25.0);
   int avoidCoin = rmCreateTypeDistanceConstraint("avoid coin", "mine", 40.0);
   int shortAvoidCoin = rmCreateTypeDistanceConstraint("short avoid coin", "mine", 10.0);
   int avoidResource = rmCreateTypeDistanceConstraint("resource avoid resource", "resource", 20.0);
   int avoidStartResource = rmCreateTypeDistanceConstraint("start resource no overlap", "resource", 10.0);

   // Impassable land constraints
   int avoidImpassableLand = rmCreateTerrainDistanceConstraint("avoid impassable land", "Land", false, 6.0);
   int shortAvoidImpassableLand = rmCreateTerrainDistanceConstraint("short avoid impassable land", "Land", false, 2.0);
   int longAvoidImpassableLand = rmCreateTerrainDistanceConstraint("long avoid impassable land", "Land", false, 10.0);

   // Unit constraints
   int avoidStartingUnits = rmCreateClassDistanceConstraint("objects avoid starting units", rmClassID("startingUnit"), 45.0);
   int shortAvoidStartingUnits = rmCreateClassDistanceConstraint("objects avoid starting units short", rmClassID("startingUnit"), 10.0);
   int avoidImportantItem = rmCreateClassDistanceConstraint("general items avoid each other", rmClassID("importantItem"), 10.0);
   int avoidNatives = rmCreateClassDistanceConstraint("objects avoid natives", rmClassID("natives"), 30.0);
   int avoidNativesLong = rmCreateClassDistanceConstraint("objects avoid natives long", rmClassID("natives"), 70.0);
   int avoidNativesExtreme = rmCreateClassDistanceConstraint("objects avoid natives extreme", rmClassID("natives"), 100.0);
   int avoidNuggets = rmCreateClassDistanceConstraint("objects avoid nuggets", rmClassID("nuggets"), 60.0);
   int shortNuggetConstraint = rmCreateTypeDistanceConstraint("avoid nugget objects", "AbstractNugget", 7.0);
   int bighornConstraint = rmCreateTypeDistanceConstraint("avoid bighorn sheep", "bighornsheep", 40.0);
   int caribouConstraint = rmCreateTypeDistanceConstraint("avoid caribous", "caribou", 40.0);

   // Additional building constraints
   int avoidOutposts = rmCreateTypeDistanceConstraint("avoid outposts", "outpost", 25.0);
   int avoidBlockhouses = rmCreateTypeDistanceConstraint("avoid blockhouses", "blockhouse", 35.0);
   int avoidWarHuts = rmCreateTypeDistanceConstraint("avoid war huts", "warhut", 35.0);

   // General constraint
   int avoidAll = rmCreateTypeDistanceConstraint("avoid all", "all", 6.0);

   // Trade route constraints
   int avoidTradeRoute = rmCreateTradeRouteDistanceConstraint("trade route", 5.0);
   int shortAvoidTradeRoute = rmCreateTradeRouteDistanceConstraint("short trade route", 3.0);
   int avoidTradeRouteFar = rmCreateTradeRouteDistanceConstraint("trade route far", 20.0);
   int avoidTradeSockets = rmCreateTypeDistanceConstraint("avoid trade sockets", "sockettraderoute", 10.0);
   int farAvoidTradeSockets = rmCreateTypeDistanceConstraint("far avoid trade sockets", "sockettraderoute", 40.0);
   int extremeAvoidTradeSockets = rmCreateTypeDistanceConstraint("extreme avoid trade sockets", "sockettraderoute", 80.0);
   int tradeRouteCircleConstraint = rmCreatePieConstraint("trade route circle onstraint", 0.5, 0.5, 0, rmZFractionToMeters(0.1), rmDegreesToRadians(0), rmDegreesToRadians(360));

   // Map generation status bar
   rmSetStatusText("",0.20);


   // Define and create ridge areas
   // (Outer areas only, inner areas are defined at the end of the script)

   // Define ridge size
   int radius = size / 2;
   int outerRidgeSize = radius * radius * 0.035;
   int innerRidgeSize = radius * radius * 0.01;

   // Outer north ridge
   int outerNorthRidgeArea = rmCreateArea("outer north ridge");
   rmSetAreaSize(outerNorthRidgeArea, rmAreaTilesToFraction(outerRidgeSize), rmAreaTilesToFraction(outerRidgeSize));
   rmSetAreaLocation(outerNorthRidgeArea, 0.8, 0.8);
   rmAddAreaInfluenceSegment(outerNorthRidgeArea, 0.9, 1.0, 0.6, 0.6);
   rmAddAreaInfluenceSegment(outerNorthRidgeArea, 1.0, 1.0, 0.6, 0.6);
   rmAddAreaInfluenceSegment(outerNorthRidgeArea, 1.0, 0.9, 0.6, 0.6);
   rmSetAreaCoherence(outerNorthRidgeArea, 0.8);
   rmSetAreaBaseHeight(outerNorthRidgeArea, 8.0);
   rmSetAreaElevationType(outerNorthRidgeArea, cElevTurbulence);
   rmSetAreaElevationVariation(outerNorthRidgeArea, 15.0);
   rmSetAreaHeightBlend(outerNorthRidgeArea, 1.5);
   rmSetAreaWarnFailure(outerNorthRidgeArea, false);
   rmSetAreaTerrainType(outerNorthRidgeArea, "patagonia\ground_glacier_pat");
   rmBuildArea(outerNorthRidgeArea);

   if ((cNumberNonGaiaPlayers != 5) && (cNumberNonGaiaPlayers != 7))
   {
      // Outer south ridge
      int outerSouthRidgeArea = rmCreateArea("outer south ridge");
      rmSetAreaSize(outerSouthRidgeArea, rmAreaTilesToFraction(outerRidgeSize), rmAreaTilesToFraction(outerRidgeSize));
      rmSetAreaLocation(outerSouthRidgeArea, 0.2, 0.2);
      rmAddAreaInfluenceSegment(outerSouthRidgeArea, 0.1, 0.0, 0.4, 0.4);
      rmAddAreaInfluenceSegment(outerSouthRidgeArea, 0.0, 0.0, 0.4, 0.4);
      rmAddAreaInfluenceSegment(outerSouthRidgeArea, 0.0, 0.1, 0.4, 0.4);
      rmSetAreaCoherence(outerSouthRidgeArea, 0.8);
      rmSetAreaBaseHeight(outerSouthRidgeArea, 8.0);
      rmSetAreaElevationType(outerSouthRidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(outerSouthRidgeArea, 15.0);
      rmSetAreaHeightBlend(outerSouthRidgeArea, 1.5);
      rmSetAreaWarnFailure(outerSouthRidgeArea, false);
      rmSetAreaTerrainType(outerSouthRidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(outerSouthRidgeArea);
   }

   if ((cNumberNonGaiaPlayers == 2) || (cNumberNonGaiaPlayers == 4) || (cNumberNonGaiaPlayers == 8))
   {
      // Outer east ridge
      int outerEastRidgeArea = rmCreateArea("outer east ridge");
      rmSetAreaSize(outerEastRidgeArea, rmAreaTilesToFraction(outerRidgeSize), rmAreaTilesToFraction(outerRidgeSize));
      rmSetAreaLocation(outerEastRidgeArea, 0.8, 0.2);
      rmAddAreaInfluenceSegment(outerEastRidgeArea, 1.0, 0.1, 0.6, 0.4);
      rmAddAreaInfluenceSegment(outerEastRidgeArea, 1.0, 0.0, 0.6, 0.4);
      rmAddAreaInfluenceSegment(outerEastRidgeArea, 0.9, 0.0, 0.6, 0.4);
      rmSetAreaCoherence(outerEastRidgeArea, 0.8);
      rmSetAreaBaseHeight(outerEastRidgeArea, 8.0);
      rmSetAreaElevationType(outerEastRidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(outerEastRidgeArea, 15.0);
      rmSetAreaHeightBlend(outerEastRidgeArea, 1.5);
      rmSetAreaWarnFailure(outerEastRidgeArea, false);
      rmSetAreaTerrainType(outerEastRidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(outerEastRidgeArea);

      // Outer west ridge
      int outerWestRidgeArea = rmCreateArea("outer west ridge");
      rmSetAreaSize(outerWestRidgeArea, rmAreaTilesToFraction(outerRidgeSize), rmAreaTilesToFraction(outerRidgeSize));
      rmSetAreaLocation(outerWestRidgeArea, 0.2, 0.8);
      rmAddAreaInfluenceSegment(outerWestRidgeArea, 0.0, 0.9, 0.4, 0.6);
      rmAddAreaInfluenceSegment(outerWestRidgeArea, 0.0, 1.0, 0.4, 0.6);
      rmAddAreaInfluenceSegment(outerWestRidgeArea, 0.1, 1.0, 0.4, 0.6);
      rmSetAreaCoherence(outerWestRidgeArea, 0.8);
      rmSetAreaBaseHeight(outerWestRidgeArea, 8.0);
      rmSetAreaElevationType(outerWestRidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(outerWestRidgeArea, 15.0);
      rmSetAreaHeightBlend(outerWestRidgeArea, 1.5);
      rmSetAreaWarnFailure(outerWestRidgeArea, false);
      rmSetAreaTerrainType(outerWestRidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(outerWestRidgeArea);
   }

   if (cNumberNonGaiaPlayers == 8)
   {
      // Outer northeast ridge
      int outerNortheastRidgeArea = rmCreateArea("outer northeast ridge");
      rmSetAreaSize(outerNortheastRidgeArea, rmAreaTilesToFraction(outerRidgeSize), rmAreaTilesToFraction(outerRidgeSize));
      rmSetAreaLocation(outerNortheastRidgeArea, 0.92, 0.5);
      rmAddAreaInfluenceSegment(outerNortheastRidgeArea, 1.0, 0.55, 0.64, 0.5);
      rmAddAreaInfluenceSegment(outerNortheastRidgeArea, 1.0, 0.5, 0.64, 0.5);
      rmAddAreaInfluenceSegment(outerNortheastRidgeArea, 1.0, 0.45, 0.64, 0.5);
      rmSetAreaCoherence(outerNortheastRidgeArea, 0.8);
      rmSetAreaBaseHeight(outerNortheastRidgeArea, 8.0);
      rmSetAreaElevationType(outerNortheastRidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(outerNortheastRidgeArea, 15.0);
      rmSetAreaHeightBlend(outerNortheastRidgeArea, 1.5);
      rmSetAreaWarnFailure(outerNortheastRidgeArea, false);
      rmSetAreaTerrainType(outerNortheastRidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(outerNortheastRidgeArea);

      // Outer northwest ridge
      int outerNorthwestRidgeArea = rmCreateArea("outer northwest ridge");
      rmSetAreaSize(outerNorthwestRidgeArea, rmAreaTilesToFraction(outerRidgeSize), rmAreaTilesToFraction(outerRidgeSize));
      rmSetAreaLocation(outerNorthwestRidgeArea, 0.5, 0.92);
      rmAddAreaInfluenceSegment(outerNorthwestRidgeArea, 0.55, 1.0, 0.5, 0.64);
      rmAddAreaInfluenceSegment(outerNorthwestRidgeArea, 0.5, 1.0, 0.5, 0.64);
      rmAddAreaInfluenceSegment(outerNorthwestRidgeArea, 0.45, 1.0, 0.5, 0.64);
      rmSetAreaCoherence(outerNorthwestRidgeArea, 0.8);
      rmSetAreaBaseHeight(outerNorthwestRidgeArea, 8.0);
      rmSetAreaElevationType(outerNorthwestRidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(outerNorthwestRidgeArea, 15.0);
      rmSetAreaHeightBlend(outerNorthwestRidgeArea, 1.5);
      rmSetAreaWarnFailure(outerNorthwestRidgeArea, false);
      rmSetAreaTerrainType(outerNorthwestRidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(outerNorthwestRidgeArea);

      // Outer southeast ridge
      int outerSoutheastRidgeArea = rmCreateArea("outer southeast ridge");
      rmSetAreaSize(outerSoutheastRidgeArea, rmAreaTilesToFraction(outerRidgeSize), rmAreaTilesToFraction(outerRidgeSize));
      rmSetAreaLocation(outerSoutheastRidgeArea, 0.5, 0.08);
      rmAddAreaInfluenceSegment(outerSoutheastRidgeArea, 0.55, 0.0, 0.5, 0.36);
      rmAddAreaInfluenceSegment(outerSoutheastRidgeArea, 0.5, 0.0, 0.5, 0.36);
      rmAddAreaInfluenceSegment(outerSoutheastRidgeArea, 0.45, 0.0, 0.5, 0.36);
      rmSetAreaCoherence(outerSoutheastRidgeArea, 0.8);
      rmSetAreaBaseHeight(outerSoutheastRidgeArea, 8.0);
      rmSetAreaElevationType(outerSoutheastRidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(outerSoutheastRidgeArea, 15.0);
      rmSetAreaHeightBlend(outerSoutheastRidgeArea, 1.5);
      rmSetAreaWarnFailure(outerSoutheastRidgeArea, false);
      rmSetAreaTerrainType(outerSoutheastRidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(outerSoutheastRidgeArea);

      // Outer southwest ridge
      int outerSouthwestRidgeArea = rmCreateArea("outer southwest ridge");
      rmSetAreaSize(outerSouthwestRidgeArea, rmAreaTilesToFraction(outerRidgeSize), rmAreaTilesToFraction(outerRidgeSize));
      rmSetAreaLocation(outerSouthwestRidgeArea, 0.08, 0.5);
      rmAddAreaInfluenceSegment(outerSouthwestRidgeArea, 0.0, 0.55, 0.36, 0.5);
      rmAddAreaInfluenceSegment(outerSouthwestRidgeArea, 0.0, 0.5, 0.36, 0.5);
      rmAddAreaInfluenceSegment(outerSouthwestRidgeArea, 0.0, 0.45, 0.36, 0.5);
      rmSetAreaCoherence(outerSouthwestRidgeArea, 0.8);
      rmSetAreaBaseHeight(outerSouthwestRidgeArea, 8.0);
      rmSetAreaElevationType(outerSouthwestRidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(outerSouthwestRidgeArea, 15.0);
      rmSetAreaHeightBlend(outerSouthwestRidgeArea, 1.5);
      rmSetAreaWarnFailure(outerSouthwestRidgeArea, false);
      rmSetAreaTerrainType(outerSouthwestRidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(outerSouthwestRidgeArea);
   }

   if ((cNumberNonGaiaPlayers == 3) || (cNumberNonGaiaPlayers == 6))
   {
      // Outer northeast ridge (3 or 6 players)
      int outerNE60RidgeArea = rmCreateArea("outer ne60 ridge");
      rmSetAreaSize(outerNE60RidgeArea, rmAreaTilesToFraction(outerRidgeSize), rmAreaTilesToFraction(outerRidgeSize));
      rmSetAreaLocation(outerNE60RidgeArea, 0.91, 0.39);
      rmAddAreaInfluenceSegment(outerNE60RidgeArea, 1.0, 0.31, 0.64, 0.46);
      rmAddAreaInfluenceSegment(outerNE60RidgeArea, 1.0, 0.36, 0.64, 0.46);
      rmAddAreaInfluenceSegment(outerNE60RidgeArea, 1.0, 0.41, 0.64, 0.46);
      rmSetAreaCoherence(outerNE60RidgeArea, 0.8);
      rmSetAreaBaseHeight(outerNE60RidgeArea, 8.0);
      rmSetAreaElevationType(outerNE60RidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(outerNE60RidgeArea, 15.0);
      rmSetAreaHeightBlend(outerNE60RidgeArea, 1.5);
      rmSetAreaWarnFailure(outerNE60RidgeArea, false);
      rmSetAreaTerrainType(outerNE60RidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(outerNE60RidgeArea);

      // Outer northwest ridge (3 or 6 players)
      int outerNW60RidgeArea = rmCreateArea("outer nw60 ridge");
      rmSetAreaSize(outerNW60RidgeArea, rmAreaTilesToFraction(outerRidgeSize), rmAreaTilesToFraction(outerRidgeSize));
      rmSetAreaLocation(outerNW60RidgeArea, 0.39, 0.91);
      rmAddAreaInfluenceSegment(outerNW60RidgeArea, 0.31, 1.0, 0.46, 0.64);
      rmAddAreaInfluenceSegment(outerNW60RidgeArea, 0.36, 1.0, 0.46, 0.64);
      rmAddAreaInfluenceSegment(outerNW60RidgeArea, 0.41, 1.0, 0.46, 0.64);
      rmSetAreaCoherence(outerNW60RidgeArea, 0.8);
      rmSetAreaBaseHeight(outerNW60RidgeArea, 8.0);
      rmSetAreaElevationType(outerNW60RidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(outerNW60RidgeArea, 15.0);
      rmSetAreaHeightBlend(outerNW60RidgeArea, 1.5);
      rmSetAreaWarnFailure(outerNW60RidgeArea, false);
      rmSetAreaTerrainType(outerNW60RidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(outerNW60RidgeArea);

      // Outer southeast ridge (3 or 6 players)
      int outerSE60RidgeArea = rmCreateArea("outer se60 ridge");
      rmSetAreaSize(outerSE60RidgeArea, rmAreaTilesToFraction(outerRidgeSize), rmAreaTilesToFraction(outerRidgeSize));
      rmSetAreaLocation(outerSE60RidgeArea, 0.61, 0.09);
      rmAddAreaInfluenceSegment(outerSE60RidgeArea, 0.59, 0.0, 0.54, 0.36);
      rmAddAreaInfluenceSegment(outerSE60RidgeArea, 0.64, 0.0, 0.54, 0.36);
      rmAddAreaInfluenceSegment(outerSE60RidgeArea, 0.69, 0.0, 0.54, 0.36);
      rmSetAreaCoherence(outerSE60RidgeArea, 0.8);
      rmSetAreaBaseHeight(outerSE60RidgeArea, 8.0);
      rmSetAreaElevationType(outerSE60RidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(outerSE60RidgeArea, 15.0);
      rmSetAreaHeightBlend(outerSE60RidgeArea, 1.5);
      rmSetAreaWarnFailure(outerSE60RidgeArea, false);
      rmSetAreaTerrainType(outerSE60RidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(outerSE60RidgeArea);

      // Outer southwest ridge (3 or 6 players)
      int outerSW60RidgeArea = rmCreateArea("outer sw60 ridge");
      rmSetAreaSize(outerSW60RidgeArea, rmAreaTilesToFraction(outerRidgeSize), rmAreaTilesToFraction(outerRidgeSize));
      rmSetAreaLocation(outerSW60RidgeArea, 0.09, 0.61);
      rmAddAreaInfluenceSegment(outerSW60RidgeArea, 0.0, 0.59, 0.36, 0.54);
      rmAddAreaInfluenceSegment(outerSW60RidgeArea, 0.0, 0.64, 0.36, 0.54);
      rmAddAreaInfluenceSegment(outerSW60RidgeArea, 0.0, 0.69, 0.36, 0.54);
      rmSetAreaCoherence(outerSW60RidgeArea, 0.8);
      rmSetAreaBaseHeight(outerSW60RidgeArea, 8.0);
      rmSetAreaElevationType(outerSW60RidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(outerSW60RidgeArea, 15.0);
      rmSetAreaHeightBlend(outerSW60RidgeArea, 1.5);
      rmSetAreaWarnFailure(outerSW60RidgeArea, false);
      rmSetAreaTerrainType(outerSW60RidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(outerSW60RidgeArea);
   }

   if (cNumberNonGaiaPlayers == 5)
   {
      // Outer northeast ridge (5 players)
      int outerNE72RidgeArea = rmCreateArea("outer ne72 ridge");
      rmSetAreaSize(outerNE72RidgeArea, rmAreaTilesToFraction(outerRidgeSize), rmAreaTilesToFraction(outerRidgeSize));
      rmSetAreaLocation(outerNE72RidgeArea, 0.88, 0.31);
      rmAddAreaInfluenceSegment(outerNE72RidgeArea, 1.0, 0.18, 0.63, 0.44);
      rmAddAreaInfluenceSegment(outerNE72RidgeArea, 1.0, 0.24, 0.63, 0.44);
      rmAddAreaInfluenceSegment(outerNE72RidgeArea, 1.0, 0.30, 0.63, 0.44);
      rmSetAreaCoherence(outerNE72RidgeArea, 0.8);
      rmSetAreaBaseHeight(outerNE72RidgeArea, 8.0);
      rmSetAreaElevationType(outerNE72RidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(outerNE72RidgeArea, 15.0);
      rmSetAreaHeightBlend(outerNE72RidgeArea, 1.5);
      rmSetAreaWarnFailure(outerNE72RidgeArea, false);
      rmSetAreaTerrainType(outerNE72RidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(outerNE72RidgeArea);

      // Outer northwest ridge (5 players)
      int outerNW72RidgeArea = rmCreateArea("outer nw72 ridge");
      rmSetAreaSize(outerNW72RidgeArea, rmAreaTilesToFraction(outerRidgeSize), rmAreaTilesToFraction(outerRidgeSize));
      rmSetAreaLocation(outerNW72RidgeArea, 0.31, 0.88);
      rmAddAreaInfluenceSegment(outerNW72RidgeArea, 0.18, 1.0, 0.44, 0.63);
      rmAddAreaInfluenceSegment(outerNW72RidgeArea, 0.24, 1.0, 0.44, 0.63);
      rmAddAreaInfluenceSegment(outerNW72RidgeArea, 0.30, 1.0, 0.44, 0.63);
      rmSetAreaCoherence(outerNW72RidgeArea, 0.8);
      rmSetAreaBaseHeight(outerNW72RidgeArea, 8.0);
      rmSetAreaElevationType(outerNW72RidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(outerNW72RidgeArea, 15.0);
      rmSetAreaHeightBlend(outerNW72RidgeArea, 1.5);
      rmSetAreaWarnFailure(outerNW72RidgeArea, false);
      rmSetAreaTerrainType(outerNW72RidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(outerNW72RidgeArea);

      // Outer southeast ridge (5 players)
      int outerSE72RidgeArea = rmCreateArea("outer se72 ridge");
      rmSetAreaSize(outerSE72RidgeArea, rmAreaTilesToFraction(outerRidgeSize), rmAreaTilesToFraction(outerRidgeSize));
      rmSetAreaLocation(outerSE72RidgeArea, 0.43, 0.08);
      rmAddAreaInfluenceSegment(outerSE72RidgeArea, 0.38, 0.0, 0.48, 0.36);
      rmAddAreaInfluenceSegment(outerSE72RidgeArea, 0.43, 0.0, 0.48, 0.36);
      rmAddAreaInfluenceSegment(outerSE72RidgeArea, 0.48, 0.0, 0.48, 0.36);
      rmSetAreaCoherence(outerSE72RidgeArea, 0.8);
      rmSetAreaBaseHeight(outerSE72RidgeArea, 8.0);
      rmSetAreaElevationType(outerSE72RidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(outerSE72RidgeArea, 15.0);
      rmSetAreaHeightBlend(outerSE72RidgeArea, 1.5);
      rmSetAreaWarnFailure(outerSE72RidgeArea, false);
      rmSetAreaTerrainType(outerSE72RidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(outerSE72RidgeArea);

      // Outer southwest ridge (5 players)
      int outerSW72RidgeArea = rmCreateArea("outer sw72 ridge");
      rmSetAreaSize(outerSW72RidgeArea, rmAreaTilesToFraction(outerRidgeSize), rmAreaTilesToFraction(outerRidgeSize));
      rmSetAreaLocation(outerSW72RidgeArea, 0.08, 0.43);
      rmAddAreaInfluenceSegment(outerSW72RidgeArea, 0.0, 0.38, 0.36, 0.48);
      rmAddAreaInfluenceSegment(outerSW72RidgeArea, 0.0, 0.43, 0.36, 0.48);
      rmAddAreaInfluenceSegment(outerSW72RidgeArea, 0.0, 0.48, 0.36, 0.48);
      rmSetAreaCoherence(outerSW72RidgeArea, 0.8);
      rmSetAreaBaseHeight(outerSW72RidgeArea, 8.0);
      rmSetAreaElevationType(outerSW72RidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(outerSW72RidgeArea, 15.0);
      rmSetAreaHeightBlend(outerSW72RidgeArea, 1.5);
      rmSetAreaWarnFailure(outerSW72RidgeArea, false);
      rmSetAreaTerrainType(outerSW72RidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(outerSW72RidgeArea);
   }

   if (cNumberNonGaiaPlayers == 7)
   {
      // Outer northeast ridge (7 players)
      int outerNE51RidgeArea = rmCreateArea("outer ne51 ridge");
      rmSetAreaSize(outerNE51RidgeArea, rmAreaTilesToFraction(outerRidgeSize), rmAreaTilesToFraction(outerRidgeSize));
      rmSetAreaLocation(outerNE51RidgeArea, 0.92, 0.45);
      rmAddAreaInfluenceSegment(outerNE51RidgeArea, 1.0, 0.39, 0.64, 0.48);
      rmAddAreaInfluenceSegment(outerNE51RidgeArea, 1.0, 0.44, 0.64, 0.48);
      rmAddAreaInfluenceSegment(outerNE51RidgeArea, 1.0, 0.49, 0.64, 0.48);
      rmSetAreaCoherence(outerNE51RidgeArea, 0.8);
      rmSetAreaBaseHeight(outerNE51RidgeArea, 8.0);
      rmSetAreaElevationType(outerNE51RidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(outerNE51RidgeArea, 15.0);
      rmSetAreaHeightBlend(outerNE51RidgeArea, 1.5);
      rmSetAreaWarnFailure(outerNE51RidgeArea, false);
      rmSetAreaTerrainType(outerNE51RidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(outerNE51RidgeArea);

      // Outer northwest ridge (7 players)
      int outerNW51RidgeArea = rmCreateArea("outer nw51 ridge");
      rmSetAreaSize(outerNW51RidgeArea, rmAreaTilesToFraction(outerRidgeSize), rmAreaTilesToFraction(outerRidgeSize));
      rmSetAreaLocation(outerNW51RidgeArea, 0.45, 0.92);
      rmAddAreaInfluenceSegment(outerNW51RidgeArea, 0.39, 1.0, 0.48, 0.64);
      rmAddAreaInfluenceSegment(outerNW51RidgeArea, 0.44, 1.0, 0.48, 0.64);
      rmAddAreaInfluenceSegment(outerNW51RidgeArea, 0.49, 1.0, 0.48, 0.64);
      rmSetAreaCoherence(outerNW51RidgeArea, 0.8);
      rmSetAreaBaseHeight(outerNW51RidgeArea, 8.0);
      rmSetAreaElevationType(outerNW51RidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(outerNW51RidgeArea, 15.0);
      rmSetAreaHeightBlend(outerNW51RidgeArea, 1.5);
      rmSetAreaWarnFailure(outerNW51RidgeArea, false);
      rmSetAreaTerrainType(outerNW51RidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(outerNW51RidgeArea);

      // Outer southeast ridge (7 players)
      int outerSE51RidgeArea = rmCreateArea("outer se51 ridge");
      rmSetAreaSize(outerSE51RidgeArea, rmAreaTilesToFraction(outerRidgeSize), rmAreaTilesToFraction(outerRidgeSize));
      rmSetAreaLocation(outerSE51RidgeArea, 0.73, 0.14);
      rmAddAreaInfluenceSegment(outerSE51RidgeArea, 0.77, 0.0, 0.58, 0.38);
      rmAddAreaInfluenceSegment(outerSE51RidgeArea, 0.83, 0.0, 0.58, 0.38);
      rmAddAreaInfluenceSegment(outerSE51RidgeArea, 0.89, 0.0, 0.58, 0.38);
      rmSetAreaCoherence(outerSE51RidgeArea, 0.8);
      rmSetAreaBaseHeight(outerSE51RidgeArea, 8.0);
      rmSetAreaElevationType(outerSE51RidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(outerSE51RidgeArea, 15.0);
      rmSetAreaHeightBlend(outerSE51RidgeArea, 1.5);
      rmSetAreaWarnFailure(outerSE51RidgeArea, false);
      rmSetAreaTerrainType(outerSE51RidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(outerSE51RidgeArea);

      // Outer southwest ridge (7 players)
      int outerSW51RidgeArea = rmCreateArea("outer sw51 ridge");
      rmSetAreaSize(outerSW51RidgeArea, rmAreaTilesToFraction(outerRidgeSize), rmAreaTilesToFraction(outerRidgeSize));
      rmSetAreaLocation(outerSW51RidgeArea, 0.14, 0.73);
      rmAddAreaInfluenceSegment(outerSW51RidgeArea, 0.0, 0.77, 0.38, 0.58);
      rmAddAreaInfluenceSegment(outerSW51RidgeArea, 0.0, 0.83, 0.38, 0.58);
      rmAddAreaInfluenceSegment(outerSW51RidgeArea, 0.0, 0.89, 0.38, 0.58);
      rmSetAreaCoherence(outerSW51RidgeArea, 0.8);
      rmSetAreaBaseHeight(outerSW51RidgeArea, 8.0);
      rmSetAreaElevationType(outerSW51RidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(outerSW51RidgeArea, 15.0);
      rmSetAreaHeightBlend(outerSW51RidgeArea, 1.5);
      rmSetAreaWarnFailure(outerSW51RidgeArea, false);
      rmSetAreaTerrainType(outerSW51RidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(outerSW51RidgeArea);

      // Outer southsoutheast ridge 
      int outerSSERidgeArea = rmCreateArea("outer sse ridge");
      rmSetAreaSize(outerSSERidgeArea, rmAreaTilesToFraction(outerRidgeSize), rmAreaTilesToFraction(outerRidgeSize));
      rmSetAreaLocation(outerSSERidgeArea, 0.36, 0.10);
      rmAddAreaInfluenceSegment(outerSSERidgeArea, 0.27, 0.0, 0.45, 0.37);
      rmAddAreaInfluenceSegment(outerSSERidgeArea, 0.33, 0.0, 0.45, 0.37);
      rmAddAreaInfluenceSegment(outerSSERidgeArea, 0.39, 0.0, 0.45, 0.37);
      rmSetAreaCoherence(outerSSERidgeArea, 0.8);
      rmSetAreaBaseHeight(outerSSERidgeArea, 8.0);
      rmSetAreaElevationType(outerSSERidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(outerSSERidgeArea, 15.0);
      rmSetAreaHeightBlend(outerSSERidgeArea, 1.5);
      rmSetAreaWarnFailure(outerSSERidgeArea, false);
      rmSetAreaTerrainType(outerSSERidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(outerSSERidgeArea);

      // Outer southsouthwest ridge (7 players)
      int outerSSWRidgeArea = rmCreateArea("outer ssw ridge");
      rmSetAreaSize(outerSSWRidgeArea, rmAreaTilesToFraction(outerRidgeSize), rmAreaTilesToFraction(outerRidgeSize));
      rmSetAreaLocation(outerSSWRidgeArea, 0.10, 0.36);
      rmAddAreaInfluenceSegment(outerSSWRidgeArea, 0.0, 0.27, 0.37, 0.45);
      rmAddAreaInfluenceSegment(outerSSWRidgeArea, 0.0, 0.33, 0.37, 0.45);
      rmAddAreaInfluenceSegment(outerSSWRidgeArea, 0.0, 0.39, 0.37, 0.45);
      rmSetAreaCoherence(outerSSWRidgeArea, 0.8);
      rmSetAreaBaseHeight(outerSSWRidgeArea, 8.0);
      rmSetAreaElevationType(outerSSWRidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(outerSSWRidgeArea, 15.0);
      rmSetAreaHeightBlend(outerSSWRidgeArea, 1.5);
      rmSetAreaWarnFailure(outerSSWRidgeArea, false);
      rmSetAreaTerrainType(outerSSWRidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(outerSSWRidgeArea);
   }

   // Map generation status bar
   rmSetStatusText("",0.30);


   // Create trade route

   // Define sockets
   int socketID = rmCreateObjectDef("sockets to dock Trade Posts");
   rmAddObjectDefItem(socketID, "SocketTradeRoute", 1, 0.0);
   rmSetObjectDefAllowOverlap(socketID, true);
   rmSetObjectDefMinDistance(socketID, 0.0);
   rmSetObjectDefMaxDistance(socketID, 12.0);
   rmAddObjectDefConstraint(socketID, shortAvoidTradeRoute);
   rmAddObjectDefConstraint(socketID, avoidImpassableLand);
   rmAddObjectDefConstraint(socketID, tradeRouteCircleConstraint);

   // Create dirt route
   int tradeRouteID = rmCreateTradeRoute();
   rmAddTradeRouteWaypoint(tradeRouteID, 0.50, 0.60);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.55, 0.59, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.59, 0.55, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.60, 0.50, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.59, 0.45, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.55, 0.41, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.50, 0.40, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.45, 0.41, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.41, 0.45, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.40, 0.50, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.41, 0.55, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.45, 0.59, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.50, 0.60, 2, 5);
   rmBuildTradeRoute(tradeRouteID, "dirt");

   // Add 4 or 6 TP sockets along the trade route
   vector socketLoc = cOriginVector;
   rmSetObjectDefTradeRouteID(socketID, tradeRouteID);
   if (cNumberNonGaiaPlayers < 5)
   {
      socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.1);
      rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);
      socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.35);
      rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);
      socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.6);
      rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);
      socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.85);
      rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);
   }
   else
   {
      socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.1);
      rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);
      socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.27);
      rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);
      socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.43);
      rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);
      socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.6);
      rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);
      socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.77);
      rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);
      socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.93);
      rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);
   }

   // Map generation status bar
   rmSetStatusText("",0.40);


   // Place players

   // Define locations for player areas, depending on number of players
   int placementRandomizer = 0;
   if (cNumberNonGaiaPlayers == 2)
   {
      placementRandomizer = rmRandInt(1,6);
      if (placementRandomizer == 1)
      {
         rmPlacePlayersLine(0.1, 0.5, 0.5, 0.9, 0, 0);
      }
      else if (placementRandomizer == 2)
      {
         rmPlacePlayersLine(0.1, 0.5, 0.5, 0.1, 0, 0);
      }
      else if (placementRandomizer == 3)
      {
         rmPlacePlayersLine(0.1, 0.5, 0.9, 0.5, 0, 0);
      }
      else if (placementRandomizer == 4)
      {
         rmPlacePlayersLine(0.5, 0.1, 0.9, 0.5, 0, 0);
      }
      else if (placementRandomizer == 5)
      {
         rmPlacePlayersLine(0.5, 0.1, 0.5, 0.9, 0, 0);
      }
      else // placementRandomizer == 6
      {
         rmPlacePlayersLine(0.5, 0.9, 0.9, 0.5, 0, 0);
      }
   }
   else if (cNumberNonGaiaPlayers == 3)
   {
      placementRandomizer = rmRandInt(1,3);
      if (placementRandomizer == 1)
      {
         rmPlacePlayer(1, 0.11, 0.39);
         rmPlacePlayersLine(0.78, 0.22, 0.60, 0.88, 0, 0);
      }
      else if (placementRandomizer == 2)
      {
         rmPlacePlayer(1, 0.78, 0.22);
         rmPlacePlayersLine(0.11, 0.39, 0.60, 0.88, 0, 0);
      }
      else // placementRandomizer == 3
      {
         rmPlacePlayer(1, 0.60, 0.88);
         rmPlacePlayersLine(0.11, 0.39, 0.78, 0.22, 0, 0);
      }
   }
   else if (cNumberNonGaiaPlayers == 4)
   {
      rmSetPlacementSection(0.25, 0.0);
      rmPlacePlayersCircular(0.42, 0.42, 0);
   }
   else if (cNumberNonGaiaPlayers == 5)
   {
      rmSetPlacementSection(0.23, 0.03);
      rmPlacePlayersCircular(0.42, 0.42, 0);
   }
   else if (cNumberNonGaiaPlayers == 6)
   {
      rmSetPlacementSection(0.20, 0.04);
      rmPlacePlayersCircular(0.42, 0.42, 0);
   }
   else if (cNumberNonGaiaPlayers == 7)
   {
      rmSetPlacementSection(0.21, 0.04);
      rmPlacePlayersCircular(0.42, 0.42, 0);
   }
   else // cNumberNonGaiaPlayers == 8
   {
      rmSetPlacementSection(0.20, 0.06);
      rmPlacePlayersCircular(0.42, 0.42, 0);
   }

   // Define player areas
   float playerFraction = rmAreaTilesToFraction(200);
   for(i=1; <cNumberPlayers)
   {
      int id = rmCreateArea("Player"+i);
      rmSetPlayerArea(i, id);
      rmSetAreaSize(id, playerFraction, playerFraction);
      rmSetAreaCoherence(id, 1.0);
      rmAddAreaToClass(id, classPlayer);
      rmSetAreaMinBlobs(id, 1);
      rmSetAreaMaxBlobs(id, 1);
      rmAddAreaConstraint(id, playerConstraint); 
      rmAddAreaConstraint(id, playerEdgeConstraint);
      rmAddAreaConstraint(id, avoidImpassableLand);
      rmSetAreaLocPlayer(id, i);
      rmSetAreaWarnFailure(id, false);
   }

   // Build all player areas
   rmBuildAllAreas();

   // Define town center (nomad start ignored here)
   int startingTC = rmCreateObjectDef("startingTC");
   rmAddObjectDefItem(startingTC, "townCenter", 1, 5.0);
   rmSetObjectDefMinDistance(startingTC, 0.0);
   rmSetObjectDefMaxDistance(startingTC, 0.0);
   rmAddObjectDefToClass(startingTC, rmClassID("player"));

   // Define additional buildings for 'nomad' game
   if (nomad == true)
   {
      // Starting barracks
      int startingBarracks = rmCreateObjectDef("starting barracks");
      rmAddObjectDefItem(startingBarracks, "barracks", 1, 5.0);
      rmSetObjectDefMinDistance(startingBarracks, 20.0);
      rmSetObjectDefMaxDistance(startingBarracks, 22.0);
      rmAddObjectDefToClass(startingBarracks, rmClassID("player"));
      rmAddObjectDefToClass(startingBarracks, rmClassID("importantItem"));
      rmAddObjectDefConstraint(startingBarracks, avoidOutposts);
      rmAddObjectDefConstraint(startingBarracks, avoidImportantItem);

      // Starting outpost
      int startingOutpost = rmCreateObjectDef("starting outpost");
      rmAddObjectDefItem(startingOutpost, "outpost", 1, 5.0);
      rmSetObjectDefMinDistance(startingOutpost, 20.0);
      rmSetObjectDefMaxDistance(startingOutpost, 22.0);
      rmAddObjectDefToClass(startingOutpost, rmClassID("player"));
      rmAddObjectDefToClass(startingOutpost, rmClassID("importantItem"));
      rmAddObjectDefConstraint(startingOutpost, avoidOutposts);
      rmAddObjectDefConstraint(startingOutpost, avoidImportantItem);

      // Starting blockhouse
      int startingBlockhouse = rmCreateObjectDef("starting blockhouse");
      rmAddObjectDefItem(startingBlockhouse, "blockhouse", 1, 5.0);
      rmSetObjectDefMinDistance(startingBlockhouse, 20.0);
      rmSetObjectDefMaxDistance(startingBlockhouse, 22.0);
      rmAddObjectDefToClass(startingBlockhouse, rmClassID("player"));
      rmAddObjectDefToClass(startingBlockhouse, rmClassID("importantItem"));
      rmAddObjectDefConstraint(startingBlockhouse, avoidBlockhouses);
      rmAddObjectDefConstraint(startingBlockhouse, avoidImportantItem);

      // Starting war hut
      int startingWarHut = rmCreateObjectDef("starting war hut");
      rmAddObjectDefItem(startingWarHut, "warhut", 1, 5.0);
      rmSetObjectDefMinDistance(startingWarHut, 20.0);
      rmSetObjectDefMaxDistance(startingWarHut, 22.0);
      rmAddObjectDefToClass(startingWarHut, rmClassID("player"));
      rmAddObjectDefToClass(startingWarHut, rmClassID("importantItem"));
      rmAddObjectDefConstraint(startingWarHut, avoidWarHuts);
      rmAddObjectDefConstraint(startingWarHut, avoidImportantItem);
   }

   // Define starting units (explorer, war chief, etc.)
   int startingUnits = rmCreateStartingUnitsObjectDef(5.0);
   rmSetObjectDefMinDistance(startingUnits, 10.0);
   rmSetObjectDefMaxDistance(startingUnits, 12.0);
   rmAddObjectDefToClass(startingUnits, rmClassID("startingUnit"));

   // Define starting mine close
   int startSilver1ID = rmCreateObjectDef("player silver close");
   rmAddObjectDefItem(startSilver1ID, "mine", 1, 0.0);
   rmSetObjectDefMinDistance(startSilver1ID, 12.0);
   rmSetObjectDefMaxDistance(startSilver1ID, 20.0);
   rmAddObjectDefConstraint(startSilver1ID, avoidStartResource);
   rmAddObjectDefConstraint(startSilver1ID, shortAvoidStartingUnits);

   // Define starting mine medium
   int startSilver2ID = rmCreateObjectDef("player silver medium");
   rmAddObjectDefItem(startSilver2ID, "mine", 1, 0.0);
   rmSetObjectDefMinDistance(startSilver2ID, 50.0);
   rmSetObjectDefMaxDistance(startSilver2ID, 60.0);
   rmAddObjectDefConstraint(startSilver2ID, avoidImpassableLand);

   // Define starting mine far
   int startSilver3ID = rmCreateObjectDef("player silver far");
   rmAddObjectDefItem(startSilver3ID, "mine", 1, 0.0);
   rmSetObjectDefMinDistance(startSilver3ID, 70.0);
   rmSetObjectDefMaxDistance(startSilver3ID, 90.0);
   rmAddObjectDefConstraint(startSilver3ID, avoidImpassableLand);

   // Define starting food close
   int startFood1ID = rmCreateObjectDef("player food close");
   rmAddObjectDefItem(startFood1ID, "bighornsheep", rmRandInt(5,6), 4.0);
   rmSetObjectDefMinDistance(startFood1ID, 12.0);
   rmSetObjectDefMaxDistance(startFood1ID, 16.0);
   rmAddObjectDefConstraint(startFood1ID, avoidStartResource);
   rmAddObjectDefConstraint(startFood1ID, avoidImpassableLand);
   rmAddObjectDefConstraint(startFood1ID, shortAvoidStartingUnits);
   rmSetObjectDefCreateHerd(startFood1ID, true);

   // Define starting food medium
   int startFood2ID = rmCreateObjectDef("player food medium");
   rmAddObjectDefItem(startFood2ID, "caribou", rmRandInt(5,7), 5.0);
   rmSetObjectDefMinDistance(startFood2ID, 40.0);
   rmSetObjectDefMaxDistance(startFood2ID, 50.0);
   rmAddObjectDefConstraint(startFood2ID, avoidStartResource);
   rmAddObjectDefConstraint(startFood2ID, avoidImpassableLand);
   rmSetObjectDefCreateHerd(startFood2ID, true);

   // Define starting trees
   int startAreaTreeID = rmCreateObjectDef("starting trees");
   rmAddObjectDefItem(startAreaTreeID, "TreeRockies", rmRandInt(3,4), 6.0);
   rmSetObjectDefMinDistance(startAreaTreeID, 20);
   rmSetObjectDefMaxDistance(startAreaTreeID, 24);
   rmAddObjectDefConstraint(startAreaTreeID, avoidStartResource);
   rmAddObjectDefConstraint(startAreaTreeID, shortAvoidImpassableLand);
   rmAddObjectDefConstraint(startAreaTreeID, shortAvoidStartingUnits);

   // Place starting units and resources for all payers 
   for(i=1; <cNumberPlayers)
   {
      rmPlaceObjectDefAtLoc(startingTC, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      if (nomad == true)
      {
         if (rmGetPlayerCiv(i) == rmGetCivID("Russians"))
         {
            rmPlaceObjectDefAtLoc(startingBlockhouse, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
            rmPlaceObjectDefAtLoc(startingBlockhouse, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
         }
         else if ((rmGetPlayerCiv(i) == rmGetCivID("XPAztec")) ||
                  (rmGetPlayerCiv(i) == rmGetCivID("XPIroquois")) ||
                  (rmGetPlayerCiv(i) == rmGetCivID("XPSioux")))
         {
            rmPlaceObjectDefAtLoc(startingWarHut, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
            rmPlaceObjectDefAtLoc(startingWarHut, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
         }
         else
         {
            rmPlaceObjectDefAtLoc(startingOutpost, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
            rmPlaceObjectDefAtLoc(startingOutpost, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
            rmPlaceObjectDefAtLoc(startingBarracks, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
         }
      }
      rmPlaceObjectDefAtLoc(startingUnits, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startSilver1ID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startSilver2ID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startSilver3ID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startFood1ID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startFood2ID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startAreaTreeID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startAreaTreeID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
   }

   // Map generation status bar
   rmSetStatusText("",0.50);


   // Define and place

   // Choose native civilizations (only on kind, either Cree or Nootka)
   int subCiv0 = -1;
   int subCiv1 = -1;
   int subCiv2 = -1;
   int subCiv3 = -1;
   int subCiv4 = -1;
   int subCiv5 = -1;
   int subCiv6 = -1;
   int subCiv7 = -1;

   float nativeChooser = rmRandFloat(0,1);
   if (nativeChooser <= 0.5)
   {
      subCiv0 = rmGetCivID("Cree");
      if (subCiv0 >= 0)
      {
         rmSetSubCiv(0, "Cree", true);
      }
      subCiv1 = rmGetCivID("Cree");
      if (subCiv1 >= 0)
      {
         rmSetSubCiv(1, "Cree", true);
      }
      subCiv2 = rmGetCivID("Cree");
      if (subCiv2 >= 0)
      {
         rmSetSubCiv(2, "Cree", true);
      }
      subCiv3 = rmGetCivID("Cree");
      if (subCiv3 >= 0)
      {
         rmSetSubCiv(3, "Cree", true);
      }
      subCiv4 = rmGetCivID("Cree");
      if (subCiv4 >= 0)
      {
         rmSetSubCiv(4, "Cree", true);
      }
      subCiv5 = rmGetCivID("Cree");
      if (subCiv5 >= 0)
      {
         rmSetSubCiv(5, "Cree", true);
      }
      subCiv6 = rmGetCivID("Cree");
      if (subCiv6 >= 0)
      {
         rmSetSubCiv(6, "Cree", true);
      }
      subCiv7 = rmGetCivID("Cree");
      if (subCiv7 >= 0)
      {
         rmSetSubCiv(7, "Cree", true);
      }
   }
   else // nativeChooser > 0.5
   {
      subCiv0 = rmGetCivID("Nootka");
      if (subCiv0 >= 0)
      {
         rmSetSubCiv(0, "Nootka", true);
      }
      subCiv1 = rmGetCivID("Nootka");
      if (subCiv1 >= 0)
      {
         rmSetSubCiv(1, "Nootka", true);
      }
      subCiv2 = rmGetCivID("Nootka");
      if (subCiv2 >= 0)
      {
         rmSetSubCiv(2, "Nootka", true);
      }
      subCiv3 = rmGetCivID("Nootka");
      if (subCiv3 >= 0)
      {
         rmSetSubCiv(3, "Nootka", true);
      }
      subCiv4 = rmGetCivID("Nootka");
      if (subCiv4 >= 0)
      {
         rmSetSubCiv(4, "Nootka", true);
      }
      subCiv5 = rmGetCivID("Nootka");
      if (subCiv5 >= 0)
      {
         rmSetSubCiv(5, "Nootka", true);
      }
      subCiv6 = rmGetCivID("Nootka");
      if (subCiv6 >= 0)
      {
         rmSetSubCiv(6, "Nootka", true);
      }
      subCiv7 = rmGetCivID("Nootka");
      if (subCiv7 >= 0)
      {
         rmSetSubCiv(7, "Nootka", true);
      }
   }

   // Define areas for native villages
   int nativeArea0Id = rmCreateArea("native area 0");
   int nativeArea1Id = rmCreateArea("native area 1");
   int nativeArea2Id = rmCreateArea("native area 2");
   int nativeArea3Id = rmCreateArea("native area 3");
   int nativeArea4Id = rmCreateArea("native area 4");
   int nativeArea5Id = rmCreateArea("native area 5");
   int nativeArea6Id = rmCreateArea("native area 6");
   int nativeArea7Id = rmCreateArea("native area 7");

   int nativeAreaSize = 0;
   int numNatives = 0;
   if (cNumberNonGaiaPlayers == 2)
   {
      nativeAreaSize = 1000;
      numNatives = 4;
   }
   else if (cNumberNonGaiaPlayers == 3)
   {
      nativeAreaSize = 800;
      numNatives = 6;
   }
   else if (cNumberNonGaiaPlayers == 4)
   {
      nativeAreaSize = 1600;
      numNatives = 4;
   }
   else if (cNumberNonGaiaPlayers == 5)
   {
      nativeAreaSize = 1800;
      numNatives = 5;
   }
   else if (cNumberNonGaiaPlayers == 6)
   {
      nativeAreaSize = 2000;
      numNatives = 6;
   }
   else if (cNumberNonGaiaPlayers == 7)
   {
      nativeAreaSize = 1600;
      numNatives = 7;
   }
   else // cNumberNonGaiaPlayers == 8
   {
      nativeAreaSize = 1200;
      numNatives = 8;
   }

   if ((cNumberNonGaiaPlayers == 2) || (cNumberNonGaiaPlayers == 4))
   {
      rmSetAreaSize(nativeArea0Id, rmAreaTilesToFraction(nativeAreaSize), rmAreaTilesToFraction(nativeAreaSize));
      rmSetAreaLocation(nativeArea0Id, 0.75, 0.5);
      rmSetAreaCoherence(nativeArea0Id, 0.5);
      rmSetAreaWarnFailure(nativeArea0Id, false);

      rmSetAreaSize(nativeArea1Id, rmAreaTilesToFraction(nativeAreaSize), rmAreaTilesToFraction(nativeAreaSize));
      rmSetAreaLocation(nativeArea1Id, 0.5, 0.25);
      rmSetAreaCoherence(nativeArea1Id, 0.5);
      rmSetAreaWarnFailure(nativeArea1Id, false);

      rmSetAreaSize(nativeArea2Id, rmAreaTilesToFraction(nativeAreaSize), rmAreaTilesToFraction(nativeAreaSize));
      rmSetAreaLocation(nativeArea2Id, 0.25, 0.5);
      rmSetAreaCoherence(nativeArea2Id, 0.5);
      rmSetAreaWarnFailure(nativeArea2Id, false);

      rmSetAreaSize(nativeArea3Id, rmAreaTilesToFraction(nativeAreaSize), rmAreaTilesToFraction(nativeAreaSize));
      rmSetAreaLocation(nativeArea3Id, 0.5, 0.75);
      rmSetAreaCoherence(nativeArea3Id, 0.5);
      rmSetAreaWarnFailure(nativeArea3Id, false);
   }
   else if ((cNumberNonGaiaPlayers == 5))
   {
      rmSetAreaSize(nativeArea0Id, rmAreaTilesToFraction(nativeAreaSize), rmAreaTilesToFraction(nativeAreaSize));
      rmSetAreaLocation(nativeArea0Id, 0.75, 0.55);
      rmSetAreaCoherence(nativeArea0Id, 0.5);
      rmSetAreaWarnFailure(nativeArea0Id, false);

      rmSetAreaSize(nativeArea1Id, rmAreaTilesToFraction(nativeAreaSize), rmAreaTilesToFraction(nativeAreaSize));
      rmSetAreaLocation(nativeArea1Id, 0.62, 0.28);
      rmSetAreaCoherence(nativeArea1Id, 0.5);
      rmSetAreaWarnFailure(nativeArea1Id, false);

      rmSetAreaSize(nativeArea2Id, rmAreaTilesToFraction(nativeAreaSize), rmAreaTilesToFraction(nativeAreaSize));
      rmSetAreaLocation(nativeArea2Id, 0.32, 0.32);
      rmSetAreaCoherence(nativeArea2Id, 0.5);
      rmSetAreaWarnFailure(nativeArea2Id, false);

      rmSetAreaSize(nativeArea3Id, rmAreaTilesToFraction(nativeAreaSize), rmAreaTilesToFraction(nativeAreaSize));
      rmSetAreaLocation(nativeArea3Id, 0.28, 0.62);
      rmSetAreaCoherence(nativeArea3Id, 0.5);
      rmSetAreaWarnFailure(nativeArea3Id, false);

      rmSetAreaSize(nativeArea4Id, rmAreaTilesToFraction(nativeAreaSize), rmAreaTilesToFraction(nativeAreaSize));
      rmSetAreaLocation(nativeArea4Id, 0.54, 0.75);
      rmSetAreaCoherence(nativeArea4Id, 0.5);
      rmSetAreaWarnFailure(nativeArea4Id, false);
   }
   else if ((cNumberNonGaiaPlayers == 3) || (cNumberNonGaiaPlayers == 6))
   {
      rmSetAreaSize(nativeArea0Id, rmAreaTilesToFraction(nativeAreaSize), rmAreaTilesToFraction(nativeAreaSize));
      rmSetAreaLocation(nativeArea0Id, 0.74, 0.57);
      rmSetAreaCoherence(nativeArea0Id, 0.5);
      rmSetAreaWarnFailure(nativeArea0Id, false);

      rmSetAreaSize(nativeArea1Id, rmAreaTilesToFraction(nativeAreaSize), rmAreaTilesToFraction(nativeAreaSize));
      rmSetAreaLocation(nativeArea1Id, 0.67, 0.32);
      rmSetAreaCoherence(nativeArea1Id, 0.5);
      rmSetAreaWarnFailure(nativeArea1Id, false);

      rmSetAreaSize(nativeArea2Id, rmAreaTilesToFraction(nativeAreaSize), rmAreaTilesToFraction(nativeAreaSize));
      rmSetAreaLocation(nativeArea2Id, 0.43, 0.27);
      rmSetAreaCoherence(nativeArea2Id, 0.5);
      rmSetAreaWarnFailure(nativeArea2Id, false);

      rmSetAreaSize(nativeArea3Id, rmAreaTilesToFraction(nativeAreaSize), rmAreaTilesToFraction(nativeAreaSize));
      rmSetAreaLocation(nativeArea3Id, 0.25, 0.42);
      rmSetAreaCoherence(nativeArea3Id, 0.5);
      rmSetAreaWarnFailure(nativeArea3Id, false);

      rmSetAreaSize(nativeArea4Id, rmAreaTilesToFraction(nativeAreaSize), rmAreaTilesToFraction(nativeAreaSize));
      rmSetAreaLocation(nativeArea4Id, 0.32, 0.68);
      rmSetAreaCoherence(nativeArea4Id, 0.5);
      rmSetAreaWarnFailure(nativeArea4Id, false);

      rmSetAreaSize(nativeArea5Id, rmAreaTilesToFraction(nativeAreaSize), rmAreaTilesToFraction(nativeAreaSize));
      rmSetAreaLocation(nativeArea5Id, 0.57, 0.75);
      rmSetAreaCoherence(nativeArea5Id, 0.5);
      rmSetAreaWarnFailure(nativeArea5Id, false);
   }
   else if (cNumberNonGaiaPlayers == 7)
   {
      rmSetAreaSize(nativeArea0Id, rmAreaTilesToFraction(nativeAreaSize), rmAreaTilesToFraction(nativeAreaSize));
      rmSetAreaLocation(nativeArea0Id, 0.73, 0.58);
      rmSetAreaCoherence(nativeArea0Id, 0.5);
      rmSetAreaWarnFailure(nativeArea0Id, false);

      rmSetAreaSize(nativeArea1Id, rmAreaTilesToFraction(nativeAreaSize), rmAreaTilesToFraction(nativeAreaSize));
      rmSetAreaLocation(nativeArea1Id, 0.71, 0.36);
      rmSetAreaCoherence(nativeArea1Id, 0.5);
      rmSetAreaWarnFailure(nativeArea1Id, false);

      rmSetAreaSize(nativeArea2Id, rmAreaTilesToFraction(nativeAreaSize), rmAreaTilesToFraction(nativeAreaSize));
      rmSetAreaLocation(nativeArea2Id, 0.53, 0.25);
      rmSetAreaCoherence(nativeArea2Id, 0.5);
      rmSetAreaWarnFailure(nativeArea2Id, false);

      rmSetAreaSize(nativeArea3Id, rmAreaTilesToFraction(nativeAreaSize), rmAreaTilesToFraction(nativeAreaSize));
      rmSetAreaLocation(nativeArea3Id, 0.32, 0.32);
      rmSetAreaCoherence(nativeArea3Id, 0.5);
      rmSetAreaWarnFailure(nativeArea3Id, false);

      rmSetAreaSize(nativeArea4Id, rmAreaTilesToFraction(nativeAreaSize), rmAreaTilesToFraction(nativeAreaSize));
      rmSetAreaLocation(nativeArea4Id, 0.25, 0.54);
      rmSetAreaCoherence(nativeArea4Id, 0.5);
      rmSetAreaWarnFailure(nativeArea4Id, false);

      rmSetAreaSize(nativeArea5Id, rmAreaTilesToFraction(nativeAreaSize), rmAreaTilesToFraction(nativeAreaSize));
      rmSetAreaLocation(nativeArea5Id, 0.36, 0.72);
      rmSetAreaCoherence(nativeArea5Id, 0.5);
      rmSetAreaWarnFailure(nativeArea5Id, false);

      rmSetAreaSize(nativeArea6Id, rmAreaTilesToFraction(nativeAreaSize), rmAreaTilesToFraction(nativeAreaSize));
      rmSetAreaLocation(nativeArea6Id, 0.59, 0.73);
      rmSetAreaCoherence(nativeArea6Id, 0.5);
      rmSetAreaWarnFailure(nativeArea6Id, false);
   }
   else // cNumberNonGaiaPlayers == 8
   {
      rmSetAreaSize(nativeArea0Id, rmAreaTilesToFraction(nativeAreaSize), rmAreaTilesToFraction(nativeAreaSize));
      rmSetAreaLocation(nativeArea0Id, 0.73, 0.60);
      rmSetAreaCoherence(nativeArea0Id, 0.5);
      rmSetAreaWarnFailure(nativeArea0Id, false);

      rmSetAreaSize(nativeArea1Id, rmAreaTilesToFraction(nativeAreaSize), rmAreaTilesToFraction(nativeAreaSize));
      rmSetAreaLocation(nativeArea1Id, 0.73, 0.40);
      rmSetAreaCoherence(nativeArea1Id, 0.5);
      rmSetAreaWarnFailure(nativeArea1Id, false);

      rmSetAreaSize(nativeArea2Id, rmAreaTilesToFraction(nativeAreaSize), rmAreaTilesToFraction(nativeAreaSize));
      rmSetAreaLocation(nativeArea2Id, 0.60, 0.27);
      rmSetAreaCoherence(nativeArea2Id, 0.5);
      rmSetAreaWarnFailure(nativeArea2Id, false);

      rmSetAreaSize(nativeArea3Id, rmAreaTilesToFraction(nativeAreaSize), rmAreaTilesToFraction(nativeAreaSize));
      rmSetAreaLocation(nativeArea3Id, 0.40, 0.27);
      rmSetAreaCoherence(nativeArea3Id, 0.5);
      rmSetAreaWarnFailure(nativeArea3Id, false);

      rmSetAreaSize(nativeArea4Id, rmAreaTilesToFraction(nativeAreaSize), rmAreaTilesToFraction(nativeAreaSize));
      rmSetAreaLocation(nativeArea4Id, 0.27, 0.40);
      rmSetAreaCoherence(nativeArea4Id, 0.5);
      rmSetAreaWarnFailure(nativeArea4Id, false);

      rmSetAreaSize(nativeArea5Id, rmAreaTilesToFraction(nativeAreaSize), rmAreaTilesToFraction(nativeAreaSize));
      rmSetAreaLocation(nativeArea5Id, 0.27, 0.60);
      rmSetAreaCoherence(nativeArea5Id, 0.5);
      rmSetAreaWarnFailure(nativeArea5Id, false);

      rmSetAreaSize(nativeArea6Id, rmAreaTilesToFraction(nativeAreaSize), rmAreaTilesToFraction(nativeAreaSize));
      rmSetAreaLocation(nativeArea6Id, 0.40, 0.73);
      rmSetAreaCoherence(nativeArea6Id, 0.5);
      rmSetAreaWarnFailure(nativeArea6Id, false);

      rmSetAreaSize(nativeArea7Id, rmAreaTilesToFraction(nativeAreaSize), rmAreaTilesToFraction(nativeAreaSize));
      rmSetAreaLocation(nativeArea7Id, 0.60, 0.73);
      rmSetAreaCoherence(nativeArea7Id, 0.5);
      rmSetAreaWarnFailure(nativeArea7Id, false);
   }

   // Build all native areas
   rmBuildAllAreas();

   // Define and place native villages
   int nativeVillage0ID = -1;
   int nativeVillage1ID = -1;
   int nativeVillage2ID = -1;
   int nativeVillage3ID = -1;
   int nativeVillage4ID = -1;
   int nativeVillage5ID = -1;
   int nativeVillage6ID = -1;
   int nativeVillage7ID = -1;

   // First village
   if (subCiv0 == rmGetCivID("Cree"))
   {
      // Cree village 0
      int creeVillage0Type = rmRandInt(1,5);
      nativeVillage0ID = rmCreateGrouping("native village 0", "native cree village "+creeVillage0Type);
      rmSetGroupingMinDistance(nativeVillage0ID, 0.0);
      rmSetGroupingMaxDistance(nativeVillage0ID, rmXFractionToMeters(0.1));
      rmAddGroupingConstraint(nativeVillage0ID, avoidImpassableLand);
      rmAddGroupingToClass(nativeVillage0ID, rmClassID("natives"));
      rmAddGroupingToClass(nativeVillage0ID, rmClassID("importantItem"));
      rmPlaceGroupingInArea(nativeVillage0ID, 0, nativeArea0Id);
   }
   else // subCiv0 == rmGetCivID("Nootka")
   {
      // Nootka village 0
      int nootkaVillage0Type = rmRandInt(1,5);
      nativeVillage0ID = rmCreateGrouping("native village 0", "native nootka village "+nootkaVillage0Type);
      rmSetGroupingMinDistance(nativeVillage0ID, 0.0);
      rmSetGroupingMaxDistance(nativeVillage0ID, rmXFractionToMeters(0.1));
      rmAddGroupingConstraint(nativeVillage0ID, avoidImpassableLand);
      rmAddGroupingToClass(nativeVillage0ID, rmClassID("natives"));
      rmAddGroupingToClass(nativeVillage0ID, rmClassID("importantItem"));
      rmPlaceGroupingInArea(nativeVillage0ID, 0, nativeArea0Id);
   }
   // Second village
   if (subCiv1 == rmGetCivID("Cree"))
   {
      // Cree village 1
      int creeVillage1Type = rmRandInt(1,5);
      nativeVillage1ID = rmCreateGrouping("native village 1", "native cree village "+creeVillage1Type);
      rmSetGroupingMinDistance(nativeVillage1ID, 0.0);
      rmSetGroupingMaxDistance(nativeVillage1ID, rmXFractionToMeters(0.1));
      rmAddGroupingConstraint(nativeVillage1ID, avoidImpassableLand);
      rmAddGroupingToClass(nativeVillage1ID, rmClassID("natives"));
      rmAddGroupingToClass(nativeVillage1ID, rmClassID("importantItem"));
      rmPlaceGroupingInArea(nativeVillage1ID, 0, nativeArea1Id);
   }
   else // (subCiv1 == rmGetCivID("Nootka"))
   {
      // Nootka village 1
      int nootkaVillage1Type = rmRandInt(1,5);
      nativeVillage1ID = rmCreateGrouping("native village 1", "native nootka village "+nootkaVillage1Type);
      rmSetGroupingMinDistance(nativeVillage1ID, 0.0);
      rmSetGroupingMaxDistance(nativeVillage1ID, rmXFractionToMeters(0.1));
      rmAddGroupingConstraint(nativeVillage1ID, avoidImpassableLand);
      rmAddGroupingToClass(nativeVillage1ID, rmClassID("natives"));
      rmAddGroupingToClass(nativeVillage1ID, rmClassID("importantItem"));
      rmPlaceGroupingInArea(nativeVillage1ID, 0, nativeArea1Id);
   }
   // Third village
   if (subCiv2 == rmGetCivID("Cree"))
   {
      // Cree village 2
      int creeVillage2Type = rmRandInt(1,5);
      nativeVillage2ID = rmCreateGrouping("native village 2", "native cree village "+creeVillage2Type);
      rmSetGroupingMinDistance(nativeVillage2ID, 0.0);
      rmSetGroupingMaxDistance(nativeVillage2ID, rmXFractionToMeters(0.1));
      rmAddGroupingConstraint(nativeVillage2ID, avoidImpassableLand);
      rmAddGroupingToClass(nativeVillage2ID, rmClassID("natives"));
      rmAddGroupingToClass(nativeVillage2ID, rmClassID("importantItem"));
      rmPlaceGroupingInArea(nativeVillage2ID, 0, nativeArea2Id);
   }
   else // (subCiv2 == rmGetCivID("Nootka"))
   {
      // Nootka village 2
      int nootkaVillage2Type = rmRandInt(1,5);
      nativeVillage2ID = rmCreateGrouping("native village 2", "native nootka village "+nootkaVillage2Type);
      rmSetGroupingMinDistance(nativeVillage2ID, 0.0);
      rmSetGroupingMaxDistance(nativeVillage2ID, rmXFractionToMeters(0.1));
      rmAddGroupingConstraint(nativeVillage2ID, avoidImpassableLand);
      rmAddGroupingToClass(nativeVillage2ID, rmClassID("natives"));
      rmAddGroupingToClass(nativeVillage2ID, rmClassID("importantItem"));
      rmPlaceGroupingInArea(nativeVillage2ID, 0, nativeArea2Id);
   }
   // Fourth village
   if (subCiv3 == rmGetCivID("Cree"))
   {
      // Cree village 3
      int creeVillage3Type = rmRandInt(1,5);
      nativeVillage3ID = rmCreateGrouping("native village 3", "native cree village "+creeVillage3Type);
      rmSetGroupingMinDistance(nativeVillage3ID, 0.0);
      rmSetGroupingMaxDistance(nativeVillage3ID, rmXFractionToMeters(0.1));
      rmAddGroupingConstraint(nativeVillage3ID, avoidImpassableLand);
      rmAddGroupingToClass(nativeVillage3ID, rmClassID("natives"));
      rmAddGroupingToClass(nativeVillage3ID, rmClassID("importantItem"));
      rmPlaceGroupingInArea(nativeVillage3ID, 0, nativeArea3Id);
   }
   else // (subCiv3 == rmGetCivID("Nootka"))
   {
      // Nootka village 3
      int nootkaVillage3Type = rmRandInt(1,5);
      nativeVillage3ID = rmCreateGrouping("native village 3", "native nootka village "+nootkaVillage3Type);
      rmSetGroupingMinDistance(nativeVillage3ID, 0.0);
      rmSetGroupingMaxDistance(nativeVillage3ID, rmXFractionToMeters(0.1));
      rmAddGroupingConstraint(nativeVillage3ID, avoidImpassableLand);
      rmAddGroupingToClass(nativeVillage3ID, rmClassID("natives"));
      rmAddGroupingToClass(nativeVillage3ID, rmClassID("importantItem"));
      rmPlaceGroupingInArea(nativeVillage3ID, 0, nativeArea3Id);
   }
   // Fifth village
   if ((cNumberNonGaiaPlayers == 3) || (cNumberNonGaiaPlayers > 4))
   {
      if (subCiv4 == rmGetCivID("Cree"))
      {
         // Cree village 4
         int creeVillage4Type = rmRandInt(1,5);
         nativeVillage4ID = rmCreateGrouping("native village 4", "native cree village "+creeVillage4Type);
         rmSetGroupingMinDistance(nativeVillage4ID, 0.0);
         rmSetGroupingMaxDistance(nativeVillage4ID, rmXFractionToMeters(0.1));
         rmAddGroupingConstraint(nativeVillage4ID, avoidImpassableLand);
         rmAddGroupingToClass(nativeVillage4ID, rmClassID("natives"));
         rmAddGroupingToClass(nativeVillage4ID, rmClassID("importantItem"));
         rmPlaceGroupingInArea(nativeVillage4ID, 0, nativeArea4Id);
      }
      else // (subCiv4 == rmGetCivID("Nootka"))
      {
         // Nootka village 4
         int nootkaVillage4Type = rmRandInt(1,5);
         nativeVillage4ID = rmCreateGrouping("native village 4", "native nootka village "+nootkaVillage4Type);
         rmSetGroupingMinDistance(nativeVillage4ID, 0.0);
         rmSetGroupingMaxDistance(nativeVillage4ID, rmXFractionToMeters(0.1));
         rmAddGroupingConstraint(nativeVillage4ID, avoidImpassableLand);
         rmAddGroupingToClass(nativeVillage4ID, rmClassID("natives"));
         rmAddGroupingToClass(nativeVillage4ID, rmClassID("importantItem"));
         rmPlaceGroupingInArea(nativeVillage4ID, 0, nativeArea4Id);
      }
   }
   // Sixth village
   if ((cNumberNonGaiaPlayers == 3) || (cNumberNonGaiaPlayers > 5))
   {
      if (subCiv5 == rmGetCivID("Cree"))
      {
         // Cree village 5
         int creeVillage5Type = rmRandInt(1,5);
         nativeVillage5ID = rmCreateGrouping("native village 5", "native cree village "+creeVillage5Type);
         rmSetGroupingMinDistance(nativeVillage5ID, 0.0);
         rmSetGroupingMaxDistance(nativeVillage5ID, rmXFractionToMeters(0.1));
         rmAddGroupingConstraint(nativeVillage5ID, avoidImpassableLand);
         rmAddGroupingToClass(nativeVillage5ID, rmClassID("natives"));
         rmAddGroupingToClass(nativeVillage5ID, rmClassID("importantItem"));
         rmPlaceGroupingInArea(nativeVillage5ID, 0, nativeArea5Id);
      }
      else // (subCiv5 == rmGetCivID("Nootka"))
      {
         // Nootka village 5
         int nootkaVillage5Type = rmRandInt(1,5);
         nativeVillage5ID = rmCreateGrouping("native village 5", "native nootka village "+nootkaVillage5Type);
         rmSetGroupingMinDistance(nativeVillage5ID, 0.0);
         rmSetGroupingMaxDistance(nativeVillage5ID, rmXFractionToMeters(0.1));
         rmAddGroupingConstraint(nativeVillage5ID, avoidImpassableLand);
         rmAddGroupingToClass(nativeVillage5ID, rmClassID("natives"));
         rmAddGroupingToClass(nativeVillage5ID, rmClassID("importantItem"));
         rmPlaceGroupingInArea(nativeVillage5ID, 0, nativeArea5Id);
      }
   }
   // Seventh village
   if (cNumberNonGaiaPlayers > 6)
   {
      if (subCiv6 == rmGetCivID("Cree"))
      {
         // Cree village 6
         int creeVillage6Type = rmRandInt(1,5);
         nativeVillage6ID = rmCreateGrouping("native village 6", "native cree village "+creeVillage6Type);
         rmSetGroupingMinDistance(nativeVillage6ID, 0.0);
         rmSetGroupingMaxDistance(nativeVillage6ID, rmXFractionToMeters(0.1));
         rmAddGroupingConstraint(nativeVillage6ID, avoidImpassableLand);
         rmAddGroupingToClass(nativeVillage6ID, rmClassID("natives"));
         rmAddGroupingToClass(nativeVillage6ID, rmClassID("importantItem"));
         rmPlaceGroupingInArea(nativeVillage6ID, 0, nativeArea6Id);
      }
      else // (subCiv6 == rmGetCivID("Nootka"))
      {
         // Nootka village 6
         int nootkaVillage6Type = rmRandInt(1,5);
         nativeVillage6ID = rmCreateGrouping("native village 6", "native nootka village "+nootkaVillage6Type);
         rmSetGroupingMinDistance(nativeVillage6ID, 0.0);
         rmSetGroupingMaxDistance(nativeVillage6ID, rmXFractionToMeters(0.1));
         rmAddGroupingConstraint(nativeVillage6ID, avoidImpassableLand);
         rmAddGroupingToClass(nativeVillage6ID, rmClassID("natives"));
         rmAddGroupingToClass(nativeVillage6ID, rmClassID("importantItem"));
         rmPlaceGroupingInArea(nativeVillage6ID, 0, nativeArea6Id);
      }
   }
   // Eighth village
   if (cNumberNonGaiaPlayers > 7)
   {
      if (subCiv7 == rmGetCivID("Cree"))
      {
         // Cree village 7
         int creeVillage7Type = rmRandInt(1,5);
         nativeVillage7ID = rmCreateGrouping("native village 7", "native cree village "+creeVillage7Type);
         rmSetGroupingMinDistance(nativeVillage7ID, 0.0);
         rmSetGroupingMaxDistance(nativeVillage7ID, rmXFractionToMeters(0.1));
         rmAddGroupingConstraint(nativeVillage7ID, avoidImpassableLand);
         rmAddGroupingToClass(nativeVillage7ID, rmClassID("natives"));
         rmAddGroupingToClass(nativeVillage7ID, rmClassID("importantItem"));
         rmPlaceGroupingInArea(nativeVillage7ID, 0, nativeArea7Id);
      }
      else // (subCiv7 == rmGetCivID("Nootka"))
      {
         // Nootka village 7
         int nootkaVillage7Type = rmRandInt(1,5);
         nativeVillage7ID = rmCreateGrouping("native village 7", "native nootka village "+nootkaVillage7Type);
         rmSetGroupingMinDistance(nativeVillage7ID, 0.0);
         rmSetGroupingMaxDistance(nativeVillage7ID, rmXFractionToMeters(0.1));
         rmAddGroupingConstraint(nativeVillage7ID, avoidImpassableLand);
         rmAddGroupingToClass(nativeVillage7ID, rmClassID("natives"));
         rmAddGroupingToClass(nativeVillage7ID, rmClassID("importantItem"));
         rmPlaceGroupingInArea(nativeVillage7ID, 0, nativeArea7Id);
      }
   }

   // Map generation status bar
   rmSetStatusText("",0.60);


   // Place mines and forests

   // Place central gold mines
   int goldCount = cNumberNonGaiaPlayers;
   if (goldCount > 6) 
   {
      goldCount = 6;
   }
   int centralMineId = rmCreateObjectDef("central mines");
   rmAddObjectDefItem(centralMineId, "minegold", 1, 0.0);
   rmSetObjectDefMinDistance(centralMineId, 0.0);
   rmSetObjectDefMaxDistance(centralMineId, rmXFractionToMeters(0.08));
   rmAddObjectDefConstraint(centralMineId, avoidImpassableLand);
   rmAddObjectDefConstraint(centralMineId, avoidGold);
   rmAddObjectDefConstraint(centralMineId, longPlayerConstraint);
   rmAddObjectDefConstraint(centralMineId, avoidAll);
   rmAddObjectDefConstraint(centralMineId, avoidTradeRoute);
   rmAddObjectDefConstraint(centralMineId, avoidTradeSockets);
   for(i=0; <goldCount)
   {
      rmPlaceObjectDefAtLoc(centralMineId, 0, 0.5, 0.5);
   }

   // Place silver mines per map quadrant
   int silverCount = cNumberNonGaiaPlayers / 2;
   // Northeastern mines
   int northeasternMineId  = rmCreateObjectDef("northeastern mines");
   rmAddObjectDefItem(northeasternMineId , "mine", 1, 0.0);
   rmSetObjectDefMinDistance(northeasternMineId , rmXFractionToMeters(0.15));
   rmSetObjectDefMaxDistance(northeasternMineId , rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(northeasternMineId , avoidImpassableLand);
   rmAddObjectDefConstraint(northeasternMineId , avoidCoin);
   rmAddObjectDefConstraint(northeasternMineId , longPlayerConstraint);
   rmAddObjectDefConstraint(northeasternMineId , avoidAll);
   rmAddObjectDefConstraint(northeasternMineId , avoidTradeRoute);
   rmAddObjectDefConstraint(northeasternMineId , avoidTradeSockets);
   rmAddObjectDefConstraint(northeasternMineId , Northeast);
   for(i=0; <silverCount)
   {
      rmPlaceObjectDefAtLoc(northeasternMineId , 0, 0.5, 0.5);
   }
   // Southeastern mines
   int southeasternMineId  = rmCreateObjectDef("southeastern mines");
   rmAddObjectDefItem(southeasternMineId , "mine", 1, 0.0);
   rmSetObjectDefMinDistance(southeasternMineId , rmXFractionToMeters(0.15));
   rmSetObjectDefMaxDistance(southeasternMineId , rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(southeasternMineId , avoidImpassableLand);
   rmAddObjectDefConstraint(southeasternMineId , avoidCoin);
   rmAddObjectDefConstraint(southeasternMineId , longPlayerConstraint);
   rmAddObjectDefConstraint(southeasternMineId , avoidAll);
   rmAddObjectDefConstraint(southeasternMineId , avoidTradeRoute);
   rmAddObjectDefConstraint(southeasternMineId , avoidTradeSockets);
   rmAddObjectDefConstraint(southeasternMineId , Southeast);
   for(i=0; <silverCount)
   {
      rmPlaceObjectDefAtLoc(southeasternMineId , 0, 0.5, 0.5);
   }
   // Southwestern mines
   int southwesternMineId  = rmCreateObjectDef("southwestern mines");
   rmAddObjectDefItem(southwesternMineId , "mine", 1, 0.0);
   rmSetObjectDefMinDistance(southwesternMineId , rmXFractionToMeters(0.15));
   rmSetObjectDefMaxDistance(southwesternMineId , rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(southwesternMineId , avoidImpassableLand);
   rmAddObjectDefConstraint(southwesternMineId , avoidCoin);
   rmAddObjectDefConstraint(southwesternMineId , longPlayerConstraint);
   rmAddObjectDefConstraint(southwesternMineId , avoidAll);
   rmAddObjectDefConstraint(southwesternMineId , avoidTradeRoute);
   rmAddObjectDefConstraint(southwesternMineId , avoidTradeSockets);
   rmAddObjectDefConstraint(southwesternMineId , Southwest);
   for(i=0; <silverCount)
   {
      rmPlaceObjectDefAtLoc(southwesternMineId , 0, 0.5, 0.5);
   }
   // Northwestern mines
   int northwesternMineId  = rmCreateObjectDef("northwestern mines");
   rmAddObjectDefItem(northwesternMineId , "mine", 1, 0.0);
   rmSetObjectDefMinDistance(northwesternMineId , rmXFractionToMeters(0.15));
   rmSetObjectDefMaxDistance(northwesternMineId , rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(northwesternMineId , avoidImpassableLand);
   rmAddObjectDefConstraint(northwesternMineId , avoidCoin);
   rmAddObjectDefConstraint(northwesternMineId , longPlayerConstraint);
   rmAddObjectDefConstraint(northwesternMineId , avoidAll);
   rmAddObjectDefConstraint(northwesternMineId , avoidTradeRoute);
   rmAddObjectDefConstraint(northwesternMineId , avoidTradeSockets);
   rmAddObjectDefConstraint(northwesternMineId , Northwest);
   for(i=0; <silverCount)
   {
      rmPlaceObjectDefAtLoc(northwesternMineId , 0, 0.5, 0.5);
   }

   // Place forests (North & South)
   int forestCount = (cNumberNonGaiaPlayers * 4) + rmRandInt(0,cNumberNonGaiaPlayers);
   int failCount = 0;
   for (i=0; <forestCount)  // Northern forests
   {
      int northForest = rmCreateArea("northforest"+i);
      rmSetAreaWarnFailure(northForest, false);
      rmSetAreaSize(northForest, rmAreaTilesToFraction(250), rmAreaTilesToFraction(400));
      rmSetAreaForestType(northForest, "rockies forest");
      rmSetAreaForestDensity(northForest, 0.8);
      rmAddAreaToClass(northForest, rmClassID("classForest"));
      rmSetAreaForestClumpiness(northForest, 0.5);
      rmSetAreaForestUnderbrush(northForest, 0.9);
      rmSetAreaBaseHeight(northForest, 2.0);
      rmSetAreaMinBlobs(northForest, 1);
      rmSetAreaMaxBlobs(northForest, 3);
      rmSetAreaMinBlobDistance(northForest, 16.0);
      rmSetAreaMaxBlobDistance(northForest, 30.0);
      rmSetAreaCoherence(northForest, 0.7);
      rmSetAreaSmoothDistance(northForest, 1);
      rmAddAreaConstraint(northForest, avoidImpassableLand);
      rmAddAreaConstraint(northForest, avoidImportantItem);
      rmAddAreaConstraint(northForest, avoidTradeRoute);
      rmAddAreaConstraint(northForest, avoidTradeSockets);
      rmAddAreaConstraint(northForest, avoidNatives);
      rmAddAreaConstraint(northForest, shortAvoidCoin);
      rmAddAreaConstraint(northForest, avoidGold);
      rmAddAreaConstraint(northForest, shortPlayerConstraint);
      rmAddAreaConstraint(northForest, forestConstraint);
      rmAddAreaConstraint(northForest, Northward);
      if(rmBuildArea(northForest)==false)
      {
         // Stop trying once we fail 5 times in a row.  
         failCount++;
         if(failCount == 5)
         {      
		break;
         }
      }
      else
      {
         failCount=0;
      }
   }
   failCount=0;
   for (i=0; <forestCount)  // Southern forests
   {
      int southForest = rmCreateArea("southforest"+i);
      rmSetAreaWarnFailure(southForest, false);
      rmSetAreaSize(southForest, rmAreaTilesToFraction(250), rmAreaTilesToFraction(400));
      rmSetAreaForestType(southForest, "rockies forest");
      rmSetAreaForestDensity(southForest, 0.8);
      rmAddAreaToClass(southForest, rmClassID("classForest"));
      rmSetAreaForestClumpiness(southForest, 0.5);
      rmSetAreaForestUnderbrush(southForest, 0.9);
      rmSetAreaBaseHeight(southForest, 2.0);
      rmSetAreaMinBlobs(southForest, 1);
      rmSetAreaMaxBlobs(southForest, 3);
      rmSetAreaMinBlobDistance(southForest, 16.0);
      rmSetAreaMaxBlobDistance(southForest, 30.0);
      rmSetAreaCoherence(southForest, 0.7);
      rmSetAreaSmoothDistance(southForest, 1);
      rmAddAreaConstraint(southForest, avoidImpassableLand);
      rmAddAreaConstraint(southForest, avoidImportantItem);
      rmAddAreaConstraint(southForest, avoidTradeRoute);
      rmAddAreaConstraint(southForest, avoidTradeSockets);
      rmAddAreaConstraint(southForest, avoidNatives);
      rmAddAreaConstraint(southForest, shortAvoidCoin);
      rmAddAreaConstraint(southForest, avoidGold);
      rmAddAreaConstraint(southForest, shortPlayerConstraint);
      rmAddAreaConstraint(southForest, forestConstraint);
      rmAddAreaConstraint(southForest, Southward);
      if(rmBuildArea(southForest)==false)
      {
         // Stop trying once we fail 5 times in a row.  
         failCount++;
         if(failCount == 5)
         {      
		break;
         }
      }
      else
      {
         failCount=0;
      }
   }

   // Map generation status bar
   rmSetStatusText("",0.70);


   // Place additional food sources

   // Place caribous
   int caribouCount = rmRandInt(4,5);
   int caribouHerdCount = cNumberNonGaiaPlayers + rmRandInt(0,cNumberNonGaiaPlayers);
   int caribouNorthID = rmCreateObjectDef("caribou herd north");
   rmAddObjectDefItem(caribouNorthID, "caribou", caribouCount, 5.0);
   rmSetObjectDefMinDistance(caribouNorthID, 0.0);
   rmSetObjectDefMaxDistance(caribouNorthID, rmXFractionToMeters(0.50));
   rmAddObjectDefConstraint(caribouNorthID, avoidImpassableLand);
   rmAddObjectDefConstraint(caribouNorthID, shortAvoidCoin);
   rmAddObjectDefConstraint(caribouNorthID, avoidGold);
   rmAddObjectDefConstraint(caribouNorthID, playerConstraint);
   rmAddObjectDefConstraint(caribouNorthID, caribouConstraint);
   rmAddObjectDefConstraint(caribouNorthID, Northward);
   rmSetObjectDefCreateHerd(caribouNorthID, true);
   for(i=0; <caribouHerdCount)
   { 
      rmPlaceObjectDefAtLoc(caribouNorthID, 0, 0.5, 0.5); // northern half of the map
   }
   int caribouSouthID = rmCreateObjectDef("caribou herd south");
   rmAddObjectDefItem(caribouSouthID, "caribou", caribouCount, 5.0);
   rmSetObjectDefMinDistance(caribouSouthID, 0.0);
   rmSetObjectDefMaxDistance(caribouSouthID, rmXFractionToMeters(0.50));
   rmAddObjectDefConstraint(caribouSouthID, avoidImpassableLand);
   rmAddObjectDefConstraint(caribouSouthID, shortAvoidCoin);
   rmAddObjectDefConstraint(caribouSouthID, avoidGold);
   rmAddObjectDefConstraint(caribouSouthID, playerConstraint);
   rmAddObjectDefConstraint(caribouSouthID, caribouConstraint);
   rmAddObjectDefConstraint(caribouSouthID, Southward);
   rmSetObjectDefCreateHerd(caribouSouthID, true);
   for(i=0; <caribouHerdCount)
   { 
      rmPlaceObjectDefAtLoc(caribouSouthID, 0, 0.5, 0.5); // southern half of the map
   }

   // Place bighorn sheep
   int bighornCount = rmRandInt(5,6);
   int bighornHerdCount = 2*cNumberNonGaiaPlayers + rmRandInt(0,cNumberNonGaiaPlayers);
   int bighornNorthID = rmCreateObjectDef("bighorn sheep herd north");
   rmAddObjectDefItem(bighornNorthID, "bighornsheep", bighornCount, 5.0);
   rmSetObjectDefMinDistance(bighornNorthID, 0.0);
   rmSetObjectDefMaxDistance(bighornNorthID, rmXFractionToMeters(0.50));
   rmAddObjectDefConstraint(bighornNorthID, avoidImpassableLand);
   rmAddObjectDefConstraint(bighornNorthID, shortAvoidCoin);
   rmAddObjectDefConstraint(bighornNorthID, avoidGold);
   rmAddObjectDefConstraint(bighornNorthID, playerConstraint);
   rmAddObjectDefConstraint(bighornNorthID, caribouConstraint);
   rmAddObjectDefConstraint(bighornNorthID, bighornConstraint);
   rmAddObjectDefConstraint(bighornNorthID, Northward);
   rmSetObjectDefCreateHerd(bighornNorthID, true);
   for(i=0; <bighornHerdCount)
   { 
      rmPlaceObjectDefAtLoc(bighornNorthID, 0, 0.5, 0.5); // northern half of the map
   }
   int bighornSouthID = rmCreateObjectDef("bighorn herd south");
   rmAddObjectDefItem(bighornSouthID, "bighornsheep", bighornCount, 5.0);
   rmSetObjectDefMinDistance(bighornSouthID, 0.0);
   rmSetObjectDefMaxDistance(bighornSouthID, rmXFractionToMeters(0.50));
   rmAddObjectDefConstraint(bighornSouthID, avoidImpassableLand);
   rmAddObjectDefConstraint(bighornSouthID, shortAvoidCoin);
   rmAddObjectDefConstraint(bighornSouthID, avoidGold);
   rmAddObjectDefConstraint(bighornSouthID, playerConstraint);
   rmAddObjectDefConstraint(bighornSouthID, caribouConstraint);
   rmAddObjectDefConstraint(bighornSouthID, bighornConstraint);
   rmAddObjectDefConstraint(bighornSouthID, Southward);
   rmSetObjectDefCreateHerd(bighornSouthID, true);
   for(i=0; <bighornHerdCount)
   { 
      rmPlaceObjectDefAtLoc(bighornSouthID, 0, 0.5, 0.5); // southern half of the map
   }

   // Map generation status bar
   rmSetStatusText("",0.80);


   // Define and place Nuggets

   // Easy nuggets (1-2 per player, close to the TC)
   int nugget1 = rmCreateObjectDef("nugget easy"); 
   rmAddObjectDefItem(nugget1, "Nugget", 1, 0.0);
   rmSetNuggetDifficulty(1, 1);
   rmAddObjectDefToClass(nugget1, rmClassID("nuggets"));
   rmAddObjectDefConstraint(nugget1, shortPlayerConstraint);
   rmAddObjectDefConstraint(nugget1, avoidImpassableLand);
   rmAddObjectDefConstraint(nugget1, avoidNuggets);
   rmAddObjectDefConstraint(nugget1, avoidTradeSockets);
   rmAddObjectDefConstraint(nugget1, avoidTradeRoute);
   rmAddObjectDefConstraint(nugget1, avoidAll);
   rmAddObjectDefConstraint(nugget1, circleConstraint);
   rmSetObjectDefMinDistance(nugget1, 30.0);
   rmSetObjectDefMaxDistance(nugget1, 50.0);
   rmPlaceObjectDefPerPlayer(nugget1, false, rmRandInt(1,2));

   // Medium nuggets (1-2 per player, further from the TC)
   int nugget2 = rmCreateObjectDef("nugget medium"); 
   rmAddObjectDefItem(nugget2, "Nugget", 1, 0.0);
   rmSetNuggetDifficulty(2, 2);
   rmAddObjectDefToClass(nugget2, rmClassID("nuggets"));
   rmSetObjectDefMinDistance(nugget2, 0.0);
   rmSetObjectDefMaxDistance(nugget2, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(nugget2, shortPlayerConstraint);
   rmAddObjectDefConstraint(nugget2, avoidImpassableLand);
   rmAddObjectDefConstraint(nugget2, avoidNuggets);
   rmAddObjectDefConstraint(nugget2, avoidTradeRoute);
   rmAddObjectDefConstraint(nugget2, circleConstraint);
   rmAddObjectDefConstraint(nugget2, avoidAll);
   rmSetObjectDefMinDistance(nugget2, 50.0);
   rmSetObjectDefMaxDistance(nugget2, 100.0);
   rmPlaceObjectDefPerPlayer(nugget2, false, rmRandInt(1,2));

   // Hard nuggets (up to 2 per player, central part of the map)
   int nugget3 = rmCreateObjectDef("nugget hard"); 
   rmAddObjectDefItem(nugget3, "Nugget", 1, 0.0);
   rmSetNuggetDifficulty(3, 3);
   rmAddObjectDefToClass(nugget3, rmClassID("nuggets"));
   rmSetObjectDefMinDistance(nugget3, 0.2);
   rmSetObjectDefMaxDistance(nugget3, rmXFractionToMeters(0.35));
   rmAddObjectDefConstraint(nugget3, shortPlayerConstraint);
   rmAddObjectDefConstraint(nugget3, avoidImpassableLand);
   rmAddObjectDefConstraint(nugget3, avoidNuggets);
   rmAddObjectDefConstraint(nugget3, avoidTradeRoute);
   rmAddObjectDefConstraint(nugget3, circleConstraint);
   rmAddObjectDefConstraint(nugget3, avoidAll);
   rmPlaceObjectDefAtLoc(nugget3, 0, 0.5, 0.5, rmRandInt(2,(2*cNumberNonGaiaPlayers)));

   // Very hard nuggets (1-2, center of the map)
   int nugget4 = rmCreateObjectDef("nugget nuts"); 
   rmAddObjectDefItem(nugget4, "Nugget", 1, 0.0);
   rmSetNuggetDifficulty(4, 4);
   rmAddObjectDefToClass(nugget4, rmClassID("nuggets"));
   rmSetObjectDefMinDistance(nugget4, 0.0);
   rmSetObjectDefMaxDistance(nugget4, rmXFractionToMeters(0.08));
   rmAddObjectDefConstraint(nugget4, longPlayerConstraint);
   rmAddObjectDefConstraint(nugget4, avoidImpassableLand);
   rmAddObjectDefConstraint(nugget4, avoidNuggets);
   rmAddObjectDefConstraint(nugget4, avoidTradeRoute);
   rmAddObjectDefConstraint(nugget4, circleConstraint);
   rmAddObjectDefConstraint(nugget4, avoidAll);
   rmPlaceObjectDefAtLoc(nugget4, 0, 0.5, 0.5, rmRandInt(1,2));

   // Map generation status bar
   rmSetStatusText("",0.90);


   // Add decoration

   // Place single trees
   int loneTree = rmCreateObjectDef("lone trees");
   int numTrees = size/10;
   rmAddObjectDefItem(loneTree, "TreeRockies", 1, 0.0);
   rmSetObjectDefMinDistance(loneTree, 0.0);
   rmSetObjectDefMaxDistance(loneTree, rmXFractionToMeters(0.50));
   rmAddObjectDefConstraint(loneTree, shortPlayerConstraint);
   rmAddObjectDefConstraint(loneTree, avoidTradeRoute);
   rmAddObjectDefConstraint(loneTree, avoidAll);
   rmPlaceObjectDefAtLoc(loneTree, 0, 0.5, 0.5, numTrees);


   // Define and create inner ridge areas for trees

   // Inner north ridge tree area
   int innerNorthRidgeTreeArea = rmCreateArea("inner north ridge trees");
   rmSetAreaSize(innerNorthRidgeTreeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
   rmSetAreaLocation(innerNorthRidgeTreeArea, 0.8, 0.8);
   rmAddAreaInfluenceSegment(innerNorthRidgeTreeArea, 0.95, 1.0, 0.6, 0.6);
   rmAddAreaInfluenceSegment(innerNorthRidgeTreeArea, 1.0, 0.95, 0.6, 0.6);
   rmSetAreaCoherence(innerNorthRidgeTreeArea, 0.8); 
   rmSetAreaBaseHeight(innerNorthRidgeTreeArea, 15);
   rmSetAreaElevationType(innerNorthRidgeTreeArea, cElevTurbulence);
   rmSetAreaElevationVariation(innerNorthRidgeTreeArea, 25.0);
   rmSetAreaHeightBlend(innerNorthRidgeTreeArea, 1.5);
   rmSetAreaWarnFailure(innerNorthRidgeTreeArea, false);
   rmSetAreaTerrainType(innerNorthRidgeTreeArea, "yukon\groundforestsnow_yuk");
   rmBuildArea(innerNorthRidgeTreeArea);

   // Inner north ridge
   int innerNorthRidgeArea = rmCreateArea("inner north ridge");
   rmSetAreaSize(innerNorthRidgeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
   rmSetAreaLocation(innerNorthRidgeArea, 0.8, 0.8);
   rmAddAreaInfluenceSegment(innerNorthRidgeArea, 0.95, 1.0, 0.6, 0.6);
   rmAddAreaInfluenceSegment(innerNorthRidgeArea, 1.0, 0.95, 0.6, 0.6);
   rmSetAreaCoherence(innerNorthRidgeArea, 0.8); 
   rmSetAreaBaseHeight(innerNorthRidgeArea, 15);
   rmSetAreaElevationType(innerNorthRidgeArea, cElevTurbulence);
   rmSetAreaElevationVariation(innerNorthRidgeArea, 25.0);
   rmSetAreaHeightBlend(innerNorthRidgeArea, 1.5);
   rmSetAreaWarnFailure(innerNorthRidgeArea, false);
   rmSetAreaTerrainType(innerNorthRidgeArea, "patagonia\ground_glacier_pat");
   rmBuildArea(innerNorthRidgeArea);

   if ((cNumberNonGaiaPlayers != 5) && (cNumberNonGaiaPlayers != 7))
   {
      // Inner south ridge tree area
      int innerSouthRidgeTreeArea = rmCreateArea("inner south ridge trees");
      rmSetAreaSize(innerSouthRidgeTreeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerSouthRidgeTreeArea, 0.2, 0.2);
      rmAddAreaInfluenceSegment(innerSouthRidgeTreeArea, 0.05, 0.0, 0.4, 0.4);
      rmAddAreaInfluenceSegment(innerSouthRidgeTreeArea, 0.0, 0.05, 0.4, 0.4);
      rmSetAreaCoherence(innerSouthRidgeTreeArea, 0.8);
      rmSetAreaBaseHeight(innerSouthRidgeTreeArea, 15);
      rmSetAreaElevationType(innerSouthRidgeTreeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerSouthRidgeTreeArea, 25.0);
      rmSetAreaHeightBlend(innerSouthRidgeTreeArea, 1.5);
      rmSetAreaWarnFailure(innerSouthRidgeTreeArea, false);
      rmSetAreaTerrainType(innerSouthRidgeTreeArea, "yukon\groundforestsnow_yuk");
      rmBuildArea(innerSouthRidgeTreeArea);

      // Inner south ridge
      int innerSouthRidgeArea = rmCreateArea("inner south ridge");
      rmSetAreaSize(innerSouthRidgeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerSouthRidgeArea, 0.2, 0.2);
      rmAddAreaInfluenceSegment(innerSouthRidgeArea, 0.05, 0.0, 0.4, 0.4);
      rmAddAreaInfluenceSegment(innerSouthRidgeArea, 0.0, 0.05, 0.4, 0.4);
      rmSetAreaCoherence(innerSouthRidgeArea, 0.8);
      rmSetAreaBaseHeight(innerSouthRidgeArea, 15);
      rmSetAreaElevationType(innerSouthRidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerSouthRidgeArea, 25.0);
      rmSetAreaHeightBlend(innerSouthRidgeArea, 1.5);
      rmSetAreaWarnFailure(innerSouthRidgeArea, false);
      rmSetAreaTerrainType(innerSouthRidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(innerSouthRidgeArea);
   }

   if ((cNumberNonGaiaPlayers == 2) || (cNumberNonGaiaPlayers == 4) || (cNumberNonGaiaPlayers == 8))
   {
      // Inner east ridge tree area
      int innerEastRidgeTreeArea = rmCreateArea("inner east ridge trees");
      rmSetAreaSize(innerEastRidgeTreeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerEastRidgeTreeArea, 0.8, 0.2);
      rmAddAreaInfluenceSegment(innerEastRidgeTreeArea, 0.95, 0.0, 0.6, 0.4);
      rmAddAreaInfluenceSegment(innerEastRidgeTreeArea, 1.0, 0.05, 0.6, 0.4);
      rmSetAreaCoherence(innerEastRidgeTreeArea, 0.8);
      rmSetAreaBaseHeight(innerEastRidgeTreeArea, 15);
      rmSetAreaElevationType(innerEastRidgeTreeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerEastRidgeTreeArea, 25.0);
      rmSetAreaHeightBlend(innerEastRidgeTreeArea, 1.5);
      rmSetAreaWarnFailure(innerEastRidgeTreeArea, false);
      rmSetAreaTerrainType(innerEastRidgeTreeArea, "yukon\groundforestsnow_yuk");
      rmBuildArea(innerEastRidgeTreeArea);

      // Inner east ridge
      int innerEastRidgeArea = rmCreateArea("inner east ridge");
      rmSetAreaSize(innerEastRidgeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerEastRidgeArea, 0.8, 0.2);
      rmAddAreaInfluenceSegment(innerEastRidgeArea, 0.95, 0.0, 0.6, 0.4);
      rmAddAreaInfluenceSegment(innerEastRidgeArea, 1.0, 0.05, 0.6, 0.4);
      rmSetAreaCoherence(innerEastRidgeArea, 0.8);
      rmSetAreaBaseHeight(innerEastRidgeArea, 15);
      rmSetAreaElevationType(innerEastRidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerEastRidgeArea, 25.0);
      rmSetAreaHeightBlend(innerEastRidgeArea, 1.5);
      rmSetAreaWarnFailure(innerEastRidgeArea, false);
      rmSetAreaTerrainType(innerEastRidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(innerEastRidgeArea);

      // Inner west ridge tree area
      int innerWestRidgeTreeArea = rmCreateArea("inner west ridge trees");
      rmSetAreaSize(innerWestRidgeTreeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerWestRidgeTreeArea, 0.2, 0.8);
      rmAddAreaInfluenceSegment(innerWestRidgeTreeArea, 0.0, 0.95, 0.4, 0.6);
      rmAddAreaInfluenceSegment(innerWestRidgeTreeArea, 0.05, 1.0, 0.4, 0.6);
      rmSetAreaCoherence(innerWestRidgeTreeArea, 0.8);
      rmSetAreaBaseHeight(innerWestRidgeTreeArea, 15);
      rmSetAreaElevationType(innerWestRidgeTreeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerWestRidgeTreeArea, 25.0);
      rmSetAreaHeightBlend(innerWestRidgeTreeArea, 1.5);
      rmSetAreaWarnFailure(innerWestRidgeTreeArea, false);
      rmSetAreaTerrainType(innerWestRidgeTreeArea, "yukon\groundforestsnow_yuk");
      rmBuildArea(innerWestRidgeTreeArea);

      // Inner west ridge
      int innerWestRidgeArea = rmCreateArea("inner west ridge");
      rmSetAreaSize(innerWestRidgeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerWestRidgeArea, 0.2, 0.8);
      rmAddAreaInfluenceSegment(innerWestRidgeArea, 0.0, 0.95, 0.4, 0.6);
      rmAddAreaInfluenceSegment(innerWestRidgeArea, 0.05, 1.0, 0.4, 0.6);
      rmSetAreaCoherence(innerWestRidgeArea, 0.8);
      rmSetAreaBaseHeight(innerWestRidgeArea, 15);
      rmSetAreaElevationType(innerWestRidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerWestRidgeArea, 25.0);
      rmSetAreaHeightBlend(innerWestRidgeArea, 1.5);
      rmSetAreaWarnFailure(innerWestRidgeArea, false);
      rmSetAreaTerrainType(innerWestRidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(innerWestRidgeArea);
   }

   if (cNumberNonGaiaPlayers == 8)
   {
      // Inner northeast ridge tree area
      int innerNortheastRidgeTreeArea = rmCreateArea("inner northeast ridge trees");
      rmSetAreaSize(innerNortheastRidgeTreeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerNortheastRidgeTreeArea, 0.92, 0.5);
      rmAddAreaInfluenceSegment(innerNortheastRidgeTreeArea, 1.0, 0.52, 0.64, 0.5);
      rmAddAreaInfluenceSegment(innerNortheastRidgeTreeArea, 1.0, 0.48, 0.64, 0.5);
      rmSetAreaCoherence(innerNortheastRidgeTreeArea, 0.8);
      rmSetAreaBaseHeight(innerNortheastRidgeTreeArea, 15);
      rmSetAreaElevationType(innerNortheastRidgeTreeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerNortheastRidgeTreeArea, 25.0);
      rmSetAreaHeightBlend(innerNortheastRidgeTreeArea, 1.5);
      rmSetAreaWarnFailure(innerNortheastRidgeTreeArea, false);
      rmSetAreaTerrainType(innerNortheastRidgeTreeArea, "yukon\groundforestsnow_yuk");
      rmBuildArea(innerNortheastRidgeTreeArea);

      // Inner northeast ridge
      int innerNortheastRidgeArea = rmCreateArea("inner northeast ridge");
      rmSetAreaSize(innerNortheastRidgeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerNortheastRidgeArea, 0.92, 0.5);
      rmAddAreaInfluenceSegment(innerNortheastRidgeArea, 1.0, 0.52, 0.64, 0.5);
      rmAddAreaInfluenceSegment(innerNortheastRidgeArea, 1.0, 0.48, 0.64, 0.5);
      rmSetAreaCoherence(innerNortheastRidgeArea, 0.8);
      rmSetAreaBaseHeight(innerNortheastRidgeArea, 15);
      rmSetAreaElevationType(innerNortheastRidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerNortheastRidgeArea, 25.0);
      rmSetAreaHeightBlend(innerNortheastRidgeArea, 1.5);
      rmSetAreaWarnFailure(innerNortheastRidgeArea, false);
      rmSetAreaTerrainType(innerNortheastRidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(innerNortheastRidgeArea);

      // Inner northwest ridge tree area
      int innerNorthwestRidgeTreeArea = rmCreateArea("inner northwest ridge trees");
      rmSetAreaSize(innerNorthwestRidgeTreeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerNorthwestRidgeTreeArea, 0.5, 0.92);
      rmAddAreaInfluenceSegment(innerNorthwestRidgeTreeArea, 0.52, 1.0, 0.5, 0.64);
      rmAddAreaInfluenceSegment(innerNorthwestRidgeTreeArea, 0.48, 1.0, 0.5, 0.64);
      rmSetAreaCoherence(innerNorthwestRidgeTreeArea, 0.8);
      rmSetAreaBaseHeight(innerNorthwestRidgeTreeArea, 15);
      rmSetAreaElevationType(innerNorthwestRidgeTreeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerNorthwestRidgeTreeArea, 25.0);
      rmSetAreaHeightBlend(innerNorthwestRidgeTreeArea, 1.5);
      rmSetAreaWarnFailure(innerNorthwestRidgeTreeArea, false);
      rmSetAreaTerrainType(innerNorthwestRidgeTreeArea, "yukon\groundforestsnow_yuk");
      rmBuildArea(innerNorthwestRidgeTreeArea);

      // Inner northwest ridge
      int innerNorthwestRidgeArea = rmCreateArea("inner northwest ridge");
      rmSetAreaSize(innerNorthwestRidgeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerNorthwestRidgeArea, 0.5, 0.92);
      rmAddAreaInfluenceSegment(innerNorthwestRidgeArea, 0.52, 1.0, 0.5, 0.64);
      rmAddAreaInfluenceSegment(innerNorthwestRidgeArea, 0.48, 1.0, 0.5, 0.64);
      rmSetAreaCoherence(innerNorthwestRidgeArea, 0.8);
      rmSetAreaBaseHeight(innerNorthwestRidgeArea, 15);
      rmSetAreaElevationType(innerNorthwestRidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerNorthwestRidgeArea, 25.0);
      rmSetAreaHeightBlend(innerNorthwestRidgeArea, 1.5);
      rmSetAreaWarnFailure(innerNorthwestRidgeArea, false);
      rmSetAreaTerrainType(innerNorthwestRidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(innerNorthwestRidgeArea);

      // Inner southeast ridge tree area
      int innerSoutheastRidgeTreeArea = rmCreateArea("inner southeast ridge trees");
      rmSetAreaSize(innerSoutheastRidgeTreeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerSoutheastRidgeTreeArea, 0.5, 0.08);
      rmAddAreaInfluenceSegment(innerSoutheastRidgeTreeArea, 0.52, 0.0, 0.5, 0.36);
      rmAddAreaInfluenceSegment(innerSoutheastRidgeTreeArea, 0.48, 0.0, 0.5, 0.36);
      rmSetAreaCoherence(innerSoutheastRidgeTreeArea, 0.8);
      rmSetAreaBaseHeight(innerSoutheastRidgeTreeArea, 15);
      rmSetAreaElevationType(innerSoutheastRidgeTreeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerSoutheastRidgeTreeArea, 25.0);
      rmSetAreaHeightBlend(innerSoutheastRidgeTreeArea, 1.5);
      rmSetAreaWarnFailure(innerSoutheastRidgeTreeArea, false);
      rmSetAreaTerrainType(innerSoutheastRidgeTreeArea, "yukon\groundforestsnow_yuk");
      rmBuildArea(innerSoutheastRidgeTreeArea);

      // Inner southeast ridge
      int innerSoutheastRidgeArea = rmCreateArea("inner southeast ridge");
      rmSetAreaSize(innerSoutheastRidgeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerSoutheastRidgeArea, 0.5, 0.08);
      rmAddAreaInfluenceSegment(innerSoutheastRidgeArea, 0.52, 0.0, 0.5, 0.36);
      rmAddAreaInfluenceSegment(innerSoutheastRidgeArea, 0.48, 0.0, 0.5, 0.36);
      rmSetAreaCoherence(innerSoutheastRidgeArea, 0.8);
      rmSetAreaBaseHeight(innerSoutheastRidgeArea, 15);
      rmSetAreaElevationType(innerSoutheastRidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerSoutheastRidgeArea, 25.0);
      rmSetAreaHeightBlend(innerSoutheastRidgeArea, 1.5);
      rmSetAreaWarnFailure(innerSoutheastRidgeArea, false);
      rmSetAreaTerrainType(innerSoutheastRidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(innerSoutheastRidgeArea);

      // Inner southwest ridge tree area
      int innerSouthwestRidgeTreeArea = rmCreateArea("inner southwest ridge trees");
      rmSetAreaSize(innerSouthwestRidgeTreeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerSouthwestRidgeTreeArea, 0.08, 0.5);
      rmAddAreaInfluenceSegment(innerSouthwestRidgeTreeArea, 0.0, 0.52, 0.36, 0.5);
      rmAddAreaInfluenceSegment(innerSouthwestRidgeTreeArea, 0.0, 0.48, 0.36, 0.5);
      rmSetAreaCoherence(innerSouthwestRidgeTreeArea, 0.8);
      rmSetAreaBaseHeight(innerSouthwestRidgeTreeArea, 15);
      rmSetAreaElevationType(innerSouthwestRidgeTreeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerSouthwestRidgeTreeArea, 25.0);
      rmSetAreaHeightBlend(innerSouthwestRidgeTreeArea, 1.5);
      rmSetAreaWarnFailure(innerSouthwestRidgeTreeArea, false);
      rmSetAreaTerrainType(innerSouthwestRidgeTreeArea, "yukon\groundforestsnow_yuk");
      rmBuildArea(innerSouthwestRidgeTreeArea);

      // Inner southwest ridge
      int innerSouthwestRidgeArea = rmCreateArea("inner southwest ridge");
      rmSetAreaSize(innerSouthwestRidgeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerSouthwestRidgeArea, 0.08, 0.5);
      rmAddAreaInfluenceSegment(innerSouthwestRidgeArea, 0.0, 0.52, 0.36, 0.5);
      rmAddAreaInfluenceSegment(innerSouthwestRidgeArea, 0.0, 0.48, 0.36, 0.5);
      rmSetAreaCoherence(innerSouthwestRidgeArea, 0.8);
      rmSetAreaBaseHeight(innerSouthwestRidgeArea, 15);
      rmSetAreaElevationType(innerSouthwestRidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerSouthwestRidgeArea, 25.0);
      rmSetAreaHeightBlend(innerSouthwestRidgeArea, 1.5);
      rmSetAreaWarnFailure(innerSouthwestRidgeArea, false);
      rmSetAreaTerrainType(innerSouthwestRidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(innerSouthwestRidgeArea);
   }

   if ((cNumberNonGaiaPlayers == 3) || (cNumberNonGaiaPlayers == 6))
   {
      // Inner northeast ridge tree area (3 or 6 players)
      int innerNE60RidgeTreeArea = rmCreateArea("inner ne60 ridge trees");
      rmSetAreaSize(innerNE60RidgeTreeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerNE60RidgeTreeArea, 0.91, 0.39);
      rmAddAreaInfluenceSegment(innerNE60RidgeTreeArea, 1.0, 0.34, 0.64, 0.46);
      rmAddAreaInfluenceSegment(innerNE60RidgeTreeArea, 1.0, 0.38, 0.64, 0.46);
      rmSetAreaCoherence(innerNE60RidgeTreeArea, 0.8);
      rmSetAreaBaseHeight(innerNE60RidgeTreeArea, 15);
      rmSetAreaElevationType(innerNE60RidgeTreeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerNE60RidgeTreeArea, 25.0);
      rmSetAreaHeightBlend(innerNE60RidgeTreeArea, 1.5);
      rmSetAreaWarnFailure(innerNE60RidgeTreeArea, false);
      rmSetAreaTerrainType(innerNE60RidgeTreeArea, "yukon\groundforestsnow_yuk");
      rmBuildArea(innerNE60RidgeTreeArea);

      // Inner northeast ridge (3 or 6 players)
      int innerNE60RidgeArea = rmCreateArea("inner ne60 ridge");
      rmSetAreaSize(innerNE60RidgeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerNE60RidgeArea, 0.91, 0.39);
      rmAddAreaInfluenceSegment(innerNE60RidgeArea, 1.0, 0.34, 0.64, 0.46);
      rmAddAreaInfluenceSegment(innerNE60RidgeArea, 1.0, 0.38, 0.64, 0.46);
      rmSetAreaCoherence(innerNE60RidgeArea, 0.8);
      rmSetAreaBaseHeight(innerNE60RidgeArea, 15);
      rmSetAreaElevationType(innerNE60RidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerNE60RidgeArea, 25.0);
      rmSetAreaHeightBlend(innerNE60RidgeArea, 1.5);
      rmSetAreaWarnFailure(innerNE60RidgeArea, false);
      rmSetAreaTerrainType(innerNE60RidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(innerNE60RidgeArea);

      // Inner northwest ridge tree area (3 or 6 players)
      int innerNW60RidgeTreeArea = rmCreateArea("inner nw60 ridge trees");
      rmSetAreaSize(innerNW60RidgeTreeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerNW60RidgeTreeArea, 0.39, 0.91);
      rmAddAreaInfluenceSegment(innerNW60RidgeTreeArea, 0.34, 1.0, 0.46, 0.64);
      rmAddAreaInfluenceSegment(innerNW60RidgeTreeArea, 0.38, 1.0, 0.46, 0.64);
      rmSetAreaCoherence(innerNW60RidgeTreeArea, 0.8);
      rmSetAreaBaseHeight(innerNW60RidgeTreeArea, 15);
      rmSetAreaElevationType(innerNW60RidgeTreeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerNW60RidgeTreeArea, 25.0);
      rmSetAreaHeightBlend(innerNW60RidgeTreeArea, 1.5);
      rmSetAreaWarnFailure(innerNW60RidgeTreeArea, false);
      rmSetAreaTerrainType(innerNW60RidgeTreeArea, "yukon\groundforestsnow_yuk");
      rmBuildArea(innerNW60RidgeTreeArea);

      // Inner northwest ridge (3 or 6 players)
      int innerNW60RidgeArea = rmCreateArea("inner nw60 ridge");
      rmSetAreaSize(innerNW60RidgeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerNW60RidgeArea, 0.39, 0.91);
      rmAddAreaInfluenceSegment(innerNW60RidgeArea, 0.34, 1.0, 0.46, 0.64);
      rmAddAreaInfluenceSegment(innerNW60RidgeArea, 0.38, 1.0, 0.46, 0.64);
      rmSetAreaCoherence(innerNW60RidgeArea, 0.8);
      rmSetAreaBaseHeight(innerNW60RidgeArea, 15);
      rmSetAreaElevationType(innerNW60RidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerNW60RidgeArea, 25.0);
      rmSetAreaHeightBlend(innerNW60RidgeArea, 1.5);
      rmSetAreaWarnFailure(innerNW60RidgeArea, false);
      rmSetAreaTerrainType(innerNW60RidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(innerNW60RidgeArea);

      // Inner southeast ridge tree area (3 or 6 players)
      int innerSE60RidgeTreeArea = rmCreateArea("inner se60 ridge trees");
      rmSetAreaSize(innerSE60RidgeTreeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerSE60RidgeTreeArea, 0.61, 0.09);
      rmAddAreaInfluenceSegment(innerSE60RidgeTreeArea, 0.62, 0.0, 0.54, 0.36);
      rmAddAreaInfluenceSegment(innerSE60RidgeTreeArea, 0.66, 0.0, 0.54, 0.36);
      rmSetAreaCoherence(innerSE60RidgeTreeArea, 0.8);
      rmSetAreaBaseHeight(innerSE60RidgeTreeArea, 15);
      rmSetAreaElevationType(innerSE60RidgeTreeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerSE60RidgeTreeArea, 25.0);
      rmSetAreaHeightBlend(innerSE60RidgeTreeArea, 1.5);
      rmSetAreaWarnFailure(innerSE60RidgeTreeArea, false);
      rmSetAreaTerrainType(innerSE60RidgeTreeArea, "yukon\groundforestsnow_yuk");
      rmBuildArea(innerSE60RidgeTreeArea);

      // Inner southeast ridge (3 or 6 players)
      int innerSE60RidgeArea = rmCreateArea("inner se60 ridge");
      rmSetAreaSize(innerSE60RidgeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerSE60RidgeArea, 0.61, 0.09);
      rmAddAreaInfluenceSegment(innerSE60RidgeArea, 0.62, 0.0, 0.54, 0.36);
      rmAddAreaInfluenceSegment(innerSE60RidgeArea, 0.66, 0.0, 0.54, 0.36);
      rmSetAreaCoherence(innerSE60RidgeArea, 0.8);
      rmSetAreaBaseHeight(innerSE60RidgeArea, 15);
      rmSetAreaElevationType(innerSE60RidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerSE60RidgeArea, 25.0);
      rmSetAreaHeightBlend(innerSE60RidgeArea, 1.5);
      rmSetAreaWarnFailure(innerSE60RidgeArea, false);
      rmSetAreaTerrainType(innerSE60RidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(innerSE60RidgeArea);

      // Inner southwest ridge tree area (3 or 6 players)
      int innerSW60RidgeTreeArea = rmCreateArea("inner sw60 ridge trees");
      rmSetAreaSize(innerSW60RidgeTreeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerSW60RidgeTreeArea, 0.09, 0.61);
      rmAddAreaInfluenceSegment(innerSW60RidgeTreeArea, 0.0, 0.62, 0.36, 0.54);
      rmAddAreaInfluenceSegment(innerSW60RidgeTreeArea, 0.0, 0.66, 0.36, 0.54);
      rmSetAreaCoherence(innerSW60RidgeTreeArea, 0.8);
      rmSetAreaBaseHeight(innerSW60RidgeTreeArea, 15);
      rmSetAreaElevationType(innerSW60RidgeTreeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerSW60RidgeTreeArea, 25.0);
      rmSetAreaHeightBlend(innerSW60RidgeTreeArea, 1.5);
      rmSetAreaWarnFailure(innerSW60RidgeTreeArea, false);
      rmSetAreaTerrainType(innerSW60RidgeTreeArea, "yukon\groundforestsnow_yuk");
      rmBuildArea(innerSW60RidgeTreeArea);

      // Inner southwest ridge (3 or 6 players)
      int innerSW60RidgeArea = rmCreateArea("inner sw60 ridge");
      rmSetAreaSize(innerSW60RidgeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerSW60RidgeArea, 0.09, 0.61);
      rmAddAreaInfluenceSegment(innerSW60RidgeArea, 0.0, 0.62, 0.36, 0.54);
      rmAddAreaInfluenceSegment(innerSW60RidgeArea, 0.0, 0.66, 0.36, 0.54);
      rmSetAreaCoherence(innerSW60RidgeArea, 0.8);
      rmSetAreaBaseHeight(innerSW60RidgeArea, 15);
      rmSetAreaElevationType(innerSW60RidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerSW60RidgeArea, 25.0);
      rmSetAreaHeightBlend(innerSW60RidgeArea, 1.5);
      rmSetAreaWarnFailure(innerSW60RidgeArea, false);
      rmSetAreaTerrainType(innerSW60RidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(innerSW60RidgeArea);
   }

   if (cNumberNonGaiaPlayers == 5)
   {
      // Inner northeast ridge tree area (5 players)
      int innerNE72RidgeTreeArea = rmCreateArea("inner ne72 ridge trees");
      rmSetAreaSize(innerNE72RidgeTreeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerNE72RidgeTreeArea, 0.88, 0.31);
      rmAddAreaInfluenceSegment(innerNE72RidgeTreeArea, 1.0, 0.22, 0.63, 0.44);
      rmAddAreaInfluenceSegment(innerNE72RidgeTreeArea, 1.0, 0.26, 0.63, 0.44);
      rmSetAreaCoherence(innerNE72RidgeTreeArea, 0.8);
      rmSetAreaBaseHeight(innerNE72RidgeTreeArea, 15);
      rmSetAreaElevationType(innerNE72RidgeTreeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerNE72RidgeTreeArea, 25.0);
      rmSetAreaHeightBlend(innerNE72RidgeTreeArea, 1.5);
      rmSetAreaWarnFailure(innerNE72RidgeTreeArea, false);
      rmSetAreaTerrainType(innerNE72RidgeTreeArea, "yukon\groundforestsnow_yuk");
      rmBuildArea(innerNE72RidgeTreeArea);

      // Inner northeast ridge (5 players)
      int innerNE72RidgeArea = rmCreateArea("inner ne72 ridge");
      rmSetAreaSize(innerNE72RidgeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerNE72RidgeArea, 0.88, 0.31);
      rmAddAreaInfluenceSegment(innerNE72RidgeArea, 1.0, 0.22, 0.63, 0.44);
      rmAddAreaInfluenceSegment(innerNE72RidgeArea, 1.0, 0.26, 0.63, 0.44);
      rmSetAreaCoherence(innerNE72RidgeArea, 0.8);
      rmSetAreaBaseHeight(innerNE72RidgeArea, 15);
      rmSetAreaElevationType(innerNE72RidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerNE72RidgeArea, 25.0);
      rmSetAreaHeightBlend(innerNE72RidgeArea, 1.5);
      rmSetAreaWarnFailure(innerNE72RidgeArea, false);
      rmSetAreaTerrainType(innerNE72RidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(innerNE72RidgeArea);

      // Inner northwest ridge tree area (5 players)
      int innerNW72RidgeTreeArea = rmCreateArea("inner nw72 ridge trees");
      rmSetAreaSize(innerNW72RidgeTreeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerNW72RidgeTreeArea, 0.31, 0.88);
      rmAddAreaInfluenceSegment(innerNW72RidgeTreeArea, 0.22, 1.0, 0.44, 0.63);
      rmAddAreaInfluenceSegment(innerNW72RidgeTreeArea, 0.26, 1.0, 0.44, 0.63);
      rmSetAreaCoherence(innerNW72RidgeTreeArea, 0.8);
      rmSetAreaBaseHeight(innerNW72RidgeTreeArea, 15);
      rmSetAreaElevationType(innerNW72RidgeTreeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerNW72RidgeTreeArea, 25.0);
      rmSetAreaHeightBlend(innerNW72RidgeTreeArea, 1.5);
      rmSetAreaWarnFailure(innerNW72RidgeTreeArea, false);
      rmSetAreaTerrainType(innerNW72RidgeTreeArea, "yukon\groundforestsnow_yuk");
      rmBuildArea(innerNW72RidgeTreeArea);

      // Inner northwest ridge (5 players)
      int innerNW72RidgeArea = rmCreateArea("inner nw72 ridge");
      rmSetAreaSize(innerNW72RidgeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerNW72RidgeArea, 0.31, 0.88);
      rmAddAreaInfluenceSegment(innerNW72RidgeArea, 0.22, 1.0, 0.44, 0.63);
      rmAddAreaInfluenceSegment(innerNW72RidgeArea, 0.26, 1.0, 0.44, 0.63);
      rmSetAreaCoherence(innerNW72RidgeArea, 0.8);
      rmSetAreaBaseHeight(innerNW72RidgeArea, 15);
      rmSetAreaElevationType(innerNW72RidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerNW72RidgeArea, 25.0);
      rmSetAreaHeightBlend(innerNW72RidgeArea, 1.5);
      rmSetAreaWarnFailure(innerNW72RidgeArea, false);
      rmSetAreaTerrainType(innerNW72RidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(innerNW72RidgeArea);

      // Inner southeast ridge tree area (5 players)
      int innerSE72RidgeTreeArea = rmCreateArea("inner se72 ridge trees");
      rmSetAreaSize(innerSE72RidgeTreeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerSE72RidgeTreeArea, 0.43, 0.08);
      rmAddAreaInfluenceSegment(innerSE72RidgeTreeArea, 0.41, 0.0, 0.48, 0.36);
      rmAddAreaInfluenceSegment(innerSE72RidgeTreeArea, 0.45, 0.0, 0.48, 0.36);
      rmSetAreaCoherence(innerSE72RidgeTreeArea, 0.8);
      rmSetAreaBaseHeight(innerSE72RidgeTreeArea, 15);
      rmSetAreaElevationType(innerSE72RidgeTreeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerSE72RidgeTreeArea, 25.0);
      rmSetAreaHeightBlend(innerSE72RidgeTreeArea, 1.5);
      rmSetAreaWarnFailure(innerSE72RidgeTreeArea, false);
      rmSetAreaTerrainType(innerSE72RidgeTreeArea, "yukon\groundforestsnow_yuk");
      rmBuildArea(innerSE72RidgeTreeArea);

      // Inner southeast ridge (5 players)
      int innerSE72RidgeArea = rmCreateArea("inner se72 ridge");
      rmSetAreaSize(innerSE72RidgeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerSE72RidgeArea, 0.43, 0.08);
      rmAddAreaInfluenceSegment(innerSE72RidgeArea, 0.41, 0.0, 0.48, 0.36);
      rmAddAreaInfluenceSegment(innerSE72RidgeArea, 0.45, 0.0, 0.48, 0.36);
      rmSetAreaCoherence(innerSE72RidgeArea, 0.8);
      rmSetAreaBaseHeight(innerSE72RidgeArea, 15);
      rmSetAreaElevationType(innerSE72RidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerSE72RidgeArea, 25.0);
      rmSetAreaHeightBlend(innerSE72RidgeArea, 1.5);
      rmSetAreaWarnFailure(innerSE72RidgeArea, false);
      rmSetAreaTerrainType(innerSE72RidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(innerSE72RidgeArea);

      // Inner southwest ridge tree area (5 players)
      int innerSW72RidgeTreeArea = rmCreateArea("inner sw72 ridge trees");
      rmSetAreaSize(innerSW72RidgeTreeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerSW72RidgeTreeArea, 0.08, 0.43);
      rmAddAreaInfluenceSegment(innerSW72RidgeTreeArea, 0.0, 0.41, 0.36, 0.48);
      rmAddAreaInfluenceSegment(innerSW72RidgeTreeArea, 0.0, 0.45, 0.36, 0.48);
      rmSetAreaCoherence(innerSW72RidgeTreeArea, 0.8);
      rmSetAreaBaseHeight(innerSW72RidgeTreeArea, 15);
      rmSetAreaElevationType(innerSW72RidgeTreeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerSW72RidgeTreeArea, 25.0);
      rmSetAreaHeightBlend(innerSW72RidgeTreeArea, 1.5);
      rmSetAreaWarnFailure(innerSW72RidgeTreeArea, false);
      rmSetAreaTerrainType(innerSW72RidgeTreeArea, "yukon\groundforestsnow_yuk");
      rmBuildArea(innerSW72RidgeTreeArea);

      // Inner southwest ridge (5 players)
      int innerSW72RidgeArea = rmCreateArea("inner sw72 ridge");
      rmSetAreaSize(innerSW72RidgeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerSW72RidgeArea, 0.08, 0.43);
      rmAddAreaInfluenceSegment(innerSW72RidgeArea, 0.0, 0.41, 0.36, 0.48);
      rmAddAreaInfluenceSegment(innerSW72RidgeArea, 0.0, 0.45, 0.36, 0.48);
      rmSetAreaCoherence(innerSW72RidgeArea, 0.8);
      rmSetAreaBaseHeight(innerSW72RidgeArea, 15);
      rmSetAreaElevationType(innerSW72RidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerSW72RidgeArea, 25.0);
      rmSetAreaHeightBlend(innerSW72RidgeArea, 1.5);
      rmSetAreaWarnFailure(innerSW72RidgeArea, false);
      rmSetAreaTerrainType(innerSW72RidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(innerSW72RidgeArea);
   }

   if (cNumberNonGaiaPlayers == 7)
   {
      // Inner northeast ridge tree area (7 players)
      int innerNE51RidgeTreeArea = rmCreateArea("inner ne51 ridge trees");
      rmSetAreaSize(innerNE51RidgeTreeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerNE51RidgeTreeArea, 0.92, 0.45);
      rmAddAreaInfluenceSegment(innerNE51RidgeTreeArea, 1.0, 0.42, 0.64, 0.48);
      rmAddAreaInfluenceSegment(innerNE51RidgeTreeArea, 1.0, 0.46, 0.64, 0.48);
      rmSetAreaCoherence(innerNE51RidgeTreeArea, 0.8);
      rmSetAreaBaseHeight(innerNE51RidgeTreeArea, 15);
      rmSetAreaElevationType(innerNE51RidgeTreeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerNE51RidgeTreeArea, 25.0);
      rmSetAreaHeightBlend(innerNE51RidgeTreeArea, 1.5);
      rmSetAreaWarnFailure(innerNE51RidgeTreeArea, false);
      rmSetAreaTerrainType(innerNE51RidgeTreeArea, "yukon\groundforestsnow_yuk");
      rmBuildArea(innerNE51RidgeTreeArea);

      // Inner northeast ridge (7 players)
      int innerNE51RidgeArea = rmCreateArea("inner ne51 ridge");
      rmSetAreaSize(innerNE51RidgeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerNE51RidgeArea, 0.92, 0.45);
      rmAddAreaInfluenceSegment(innerNE51RidgeArea, 1.0, 0.42, 0.64, 0.48);
      rmAddAreaInfluenceSegment(innerNE51RidgeArea, 1.0, 0.46, 0.64, 0.48);
      rmSetAreaCoherence(innerNE51RidgeArea, 0.8);
      rmSetAreaBaseHeight(innerNE51RidgeArea, 15);
      rmSetAreaElevationType(innerNE51RidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerNE51RidgeArea, 25.0);
      rmSetAreaHeightBlend(innerNE51RidgeArea, 1.5);
      rmSetAreaWarnFailure(innerNE51RidgeArea, false);
      rmSetAreaTerrainType(innerNE51RidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(innerNE51RidgeArea);

      // Inner northwest ridge tree area (7 players)
      int innerNW51RidgeTreeArea = rmCreateArea("inner nw51 ridge trees");
      rmSetAreaSize(innerNW51RidgeTreeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerNW51RidgeTreeArea, 0.45, 0.92);
      rmAddAreaInfluenceSegment(innerNW51RidgeTreeArea, 0.42, 1.0, 0.48, 0.64);
      rmAddAreaInfluenceSegment(innerNW51RidgeTreeArea, 0.46, 1.0, 0.48, 0.64);
      rmSetAreaCoherence(innerNW51RidgeTreeArea, 0.8);
      rmSetAreaBaseHeight(innerNW51RidgeTreeArea, 15);
      rmSetAreaElevationType(innerNW51RidgeTreeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerNW51RidgeTreeArea, 25.0);
      rmSetAreaHeightBlend(innerNW51RidgeTreeArea, 1.5);
      rmSetAreaWarnFailure(innerNW51RidgeTreeArea, false);
      rmSetAreaTerrainType(innerNW51RidgeTreeArea, "yukon\groundforestsnow_yuk");
      rmBuildArea(innerNW51RidgeTreeArea);

      // Inner northwest ridge (7 players)
      int innerNW51RidgeArea = rmCreateArea("inner nw51 ridge");
      rmSetAreaSize(innerNW51RidgeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerNW51RidgeArea, 0.45, 0.92);
      rmAddAreaInfluenceSegment(innerNW51RidgeArea, 0.42, 1.0, 0.48, 0.64);
      rmAddAreaInfluenceSegment(innerNW51RidgeArea, 0.46, 1.0, 0.48, 0.64);
      rmSetAreaCoherence(innerNW51RidgeArea, 0.8);
      rmSetAreaBaseHeight(innerNW51RidgeArea, 15);
      rmSetAreaElevationType(innerNW51RidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerNW51RidgeArea, 25.0);
      rmSetAreaHeightBlend(innerNW51RidgeArea, 1.5);
      rmSetAreaWarnFailure(innerNW51RidgeArea, false);
      rmSetAreaTerrainType(innerNW51RidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(innerNW51RidgeArea);

      // Inner southeast ridge tree area (7 players)
      int innerSE51RidgeTreeArea = rmCreateArea("inner se51 ridge trees");
      rmSetAreaSize(innerSE51RidgeTreeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerSE51RidgeTreeArea, 0.73, 0.14);
      rmAddAreaInfluenceSegment(innerSE51RidgeTreeArea, 0.81, 0.0, 0.58, 0.38);
      rmAddAreaInfluenceSegment(innerSE51RidgeTreeArea, 0.85, 0.0, 0.58, 0.38);
      rmSetAreaCoherence(innerSE51RidgeTreeArea, 0.8);
      rmSetAreaBaseHeight(innerSE51RidgeTreeArea, 15);
      rmSetAreaElevationType(innerSE51RidgeTreeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerSE51RidgeTreeArea, 25.0);
      rmSetAreaHeightBlend(innerSE51RidgeTreeArea, 1.5);
      rmSetAreaWarnFailure(innerSE51RidgeTreeArea, false);
      rmSetAreaTerrainType(innerSE51RidgeTreeArea, "yukon\groundforestsnow_yuk");
      rmBuildArea(innerSE51RidgeTreeArea);

      // Inner southeast ridge (7 players)
      int innerSE51RidgeArea = rmCreateArea("inner se51 ridge");
      rmSetAreaSize(innerSE51RidgeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerSE51RidgeArea, 0.73, 0.14);
      rmAddAreaInfluenceSegment(innerSE51RidgeArea, 0.81, 0.0, 0.58, 0.38);
      rmAddAreaInfluenceSegment(innerSE51RidgeArea, 0.85, 0.0, 0.58, 0.38);
      rmSetAreaCoherence(innerSE51RidgeArea, 0.8);
      rmSetAreaBaseHeight(innerSE51RidgeArea, 15);
      rmSetAreaElevationType(innerSE51RidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerSE51RidgeArea, 25.0);
      rmSetAreaHeightBlend(innerSE51RidgeArea, 1.5);
      rmSetAreaWarnFailure(innerSE51RidgeArea, false);
      rmSetAreaTerrainType(innerSE51RidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(innerSE51RidgeArea);

      // Inner southwest ridge tree area (7 players)
      int innerSW51RidgeTreeArea = rmCreateArea("inner sw51 ridge trees");
      rmSetAreaSize(innerSW51RidgeTreeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerSW51RidgeTreeArea, 0.14, 0.73);
      rmAddAreaInfluenceSegment(innerSW51RidgeTreeArea, 0.0, 0.81, 0.38, 0.58);
      rmAddAreaInfluenceSegment(innerSW51RidgeTreeArea, 0.0, 0.85, 0.38, 0.58);
      rmSetAreaCoherence(innerSW51RidgeTreeArea, 0.8);
      rmSetAreaBaseHeight(innerSW51RidgeTreeArea, 15);
      rmSetAreaElevationType(innerSW51RidgeTreeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerSW51RidgeTreeArea, 25.0);
      rmSetAreaHeightBlend(innerSW51RidgeTreeArea, 1.5);
      rmSetAreaWarnFailure(innerSW51RidgeTreeArea, false);
      rmSetAreaTerrainType(innerSW51RidgeTreeArea, "yukon\groundforestsnow_yuk");
      rmBuildArea(innerSW51RidgeTreeArea);

      // Inner southwest ridge (7 players)
      int innerSW51RidgeArea = rmCreateArea("inner sw51 ridge");
      rmSetAreaSize(innerSW51RidgeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerSW51RidgeArea, 0.14, 0.73);
      rmAddAreaInfluenceSegment(innerSW51RidgeArea, 0.0, 0.81, 0.38, 0.58);
      rmAddAreaInfluenceSegment(innerSW51RidgeArea, 0.0, 0.85, 0.38, 0.58);
      rmSetAreaCoherence(innerSW51RidgeArea, 0.8);
      rmSetAreaBaseHeight(innerSW51RidgeArea, 15);
      rmSetAreaElevationType(innerSW51RidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerSW51RidgeArea, 25.0);
      rmSetAreaHeightBlend(innerSW51RidgeArea, 1.5);
      rmSetAreaWarnFailure(innerSW51RidgeArea, false);
      rmSetAreaTerrainType(innerSW51RidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(innerSW51RidgeArea);

      // Inner southsoutheast ridge tree area (7 players)
      int innerSSERidgeTreeArea = rmCreateArea("inner sse ridge trees");
      rmSetAreaSize(innerSSERidgeTreeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerSSERidgeTreeArea, 0.36, 0.10);
      rmAddAreaInfluenceSegment(innerSSERidgeTreeArea, 0.31, 0.0, 0.45, 0.37);
      rmAddAreaInfluenceSegment(innerSSERidgeTreeArea, 0.35, 0.0, 0.45, 0.37);
      rmSetAreaCoherence(innerSSERidgeTreeArea, 0.8);
      rmSetAreaBaseHeight(innerSSERidgeTreeArea, 15);
      rmSetAreaElevationType(innerSSERidgeTreeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerSSERidgeTreeArea, 25.0);
      rmSetAreaHeightBlend(innerSSERidgeTreeArea, 1.5);
      rmSetAreaWarnFailure(innerSSERidgeTreeArea, false);
      rmSetAreaTerrainType(innerSSERidgeTreeArea, "yukon\groundforestsnow_yuk");
      rmBuildArea(innerSSERidgeTreeArea);

      // Inner southsoutheast ridge (7 players)
      int innerSSERidgeArea = rmCreateArea("inner sse ridge");
      rmSetAreaSize(innerSSERidgeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerSSERidgeArea, 0.36, 0.10);
      rmAddAreaInfluenceSegment(innerSSERidgeArea, 0.31, 0.0, 0.45, 0.37);
      rmAddAreaInfluenceSegment(innerSSERidgeArea, 0.35, 0.0, 0.45, 0.37);
      rmSetAreaCoherence(innerSSERidgeArea, 0.8);
      rmSetAreaBaseHeight(innerSSERidgeArea, 15);
      rmSetAreaElevationType(innerSSERidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerSSERidgeArea, 25.0);
      rmSetAreaHeightBlend(innerSSERidgeArea, 1.5);
      rmSetAreaWarnFailure(innerSSERidgeArea, false);
      rmSetAreaTerrainType(innerSSERidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(innerSSERidgeArea);

      // Inner southsouthwest ridge tree area (7 players)
      int innerSSWRidgeTreeArea = rmCreateArea("inner ssw ridge trees");
      rmSetAreaSize(innerSSWRidgeTreeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerSSWRidgeTreeArea, 0.10, 0.36);
      rmAddAreaInfluenceSegment(innerSSWRidgeTreeArea, 0.0, 0.31, 0.37, 0.45);
      rmAddAreaInfluenceSegment(innerSSWRidgeTreeArea, 0.0, 0.35, 0.37, 0.45);
      rmSetAreaCoherence(innerSSWRidgeTreeArea, 0.8);
      rmSetAreaBaseHeight(innerSSWRidgeTreeArea, 15);
      rmSetAreaElevationType(innerSSWRidgeTreeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerSSWRidgeTreeArea, 25.0);
      rmSetAreaHeightBlend(innerSSWRidgeTreeArea, 1.5);
      rmSetAreaWarnFailure(innerSSWRidgeTreeArea, false);
      rmSetAreaTerrainType(innerSSWRidgeTreeArea, "yukon\groundforestsnow_yuk");
      rmBuildArea(innerSSWRidgeTreeArea);

      // Inner southsouthwest ridge (7 players)
      int innerSSWRidgeArea = rmCreateArea("inner ssw ridge");
      rmSetAreaSize(innerSSWRidgeArea, rmAreaTilesToFraction(innerRidgeSize), rmAreaTilesToFraction(innerRidgeSize));
      rmSetAreaLocation(innerSSWRidgeArea, 0.10, 0.36);
      rmAddAreaInfluenceSegment(innerSSWRidgeArea, 0.0, 0.31, 0.37, 0.45);
      rmAddAreaInfluenceSegment(innerSSWRidgeArea, 0.0, 0.35, 0.37, 0.45);
      rmSetAreaCoherence(innerSSWRidgeArea, 0.8);
      rmSetAreaBaseHeight(innerSSWRidgeArea, 15);
      rmSetAreaElevationType(innerSSWRidgeArea, cElevTurbulence);
      rmSetAreaElevationVariation(innerSSWRidgeArea, 25.0);
      rmSetAreaHeightBlend(innerSSWRidgeArea, 1.5);
      rmSetAreaWarnFailure(innerSSWRidgeArea, false);
      rmSetAreaTerrainType(innerSSWRidgeArea, "patagonia\ground_glacier_pat");
      rmBuildArea(innerSSWRidgeArea);
   }

   // Place ridge trees
   int ridgeTree = rmCreateObjectDef("ridge trees");
   numTrees = size/5;
   rmAddObjectDefItem(ridgeTree , "TreeRockiesSnow", 1, 0.0);
   rmPlaceObjectDefInArea(ridgeTree , 0, innerNorthRidgeTreeArea, numTrees);
   if ((cNumberNonGaiaPlayers != 5) && (cNumberNonGaiaPlayers != 7))
   {
      rmPlaceObjectDefInArea(ridgeTree , 0, innerSouthRidgeTreeArea, numTrees);
   }
   if ((cNumberNonGaiaPlayers == 2) || (cNumberNonGaiaPlayers == 4) || (cNumberNonGaiaPlayers == 8))
   {
      rmPlaceObjectDefInArea(ridgeTree , 0, innerEastRidgeTreeArea, numTrees);
      rmPlaceObjectDefInArea(ridgeTree , 0, innerWestRidgeTreeArea, numTrees);
   }
   if (cNumberNonGaiaPlayers == 8)
   {
      rmPlaceObjectDefInArea(ridgeTree , 0, innerNortheastRidgeTreeArea, numTrees);
      rmPlaceObjectDefInArea(ridgeTree , 0, innerNorthwestRidgeTreeArea, numTrees);
      rmPlaceObjectDefInArea(ridgeTree , 0, innerSoutheastRidgeTreeArea, numTrees);
      rmPlaceObjectDefInArea(ridgeTree , 0, innerSouthwestRidgeTreeArea, numTrees);
   }
   if ((cNumberNonGaiaPlayers == 3) || (cNumberNonGaiaPlayers == 6))
   {
      rmPlaceObjectDefInArea(ridgeTree , 0, innerNE60RidgeTreeArea, numTrees);
      rmPlaceObjectDefInArea(ridgeTree , 0, innerNW60RidgeTreeArea, numTrees);
      rmPlaceObjectDefInArea(ridgeTree , 0, innerSE60RidgeTreeArea, numTrees);
      rmPlaceObjectDefInArea(ridgeTree , 0, innerSW60RidgeTreeArea, numTrees);
   }
   if (cNumberNonGaiaPlayers == 5)
   {
      rmPlaceObjectDefInArea(ridgeTree , 0, innerNE72RidgeTreeArea, numTrees);
      rmPlaceObjectDefInArea(ridgeTree , 0, innerNW72RidgeTreeArea, numTrees);
      rmPlaceObjectDefInArea(ridgeTree , 0, innerSE72RidgeTreeArea, numTrees);
      rmPlaceObjectDefInArea(ridgeTree , 0, innerSW72RidgeTreeArea, numTrees);
   }
   if (cNumberNonGaiaPlayers == 7)
   {
      rmPlaceObjectDefInArea(ridgeTree , 0, innerNE51RidgeTreeArea, numTrees);
      rmPlaceObjectDefInArea(ridgeTree , 0, innerNW51RidgeTreeArea, numTrees);
      rmPlaceObjectDefInArea(ridgeTree , 0, innerSE51RidgeTreeArea, numTrees);
      rmPlaceObjectDefInArea(ridgeTree , 0, innerSW51RidgeTreeArea, numTrees);
      rmPlaceObjectDefInArea(ridgeTree , 0, innerSSERidgeTreeArea, numTrees);
      rmPlaceObjectDefInArea(ridgeTree , 0, innerSSWRidgeTreeArea, numTrees);
   }

   // Map generation status bar
   rmSetStatusText("",1.0);
}  
